@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from gettext import gettext as _
 import logging
 import os
 import pathlib
@@ -13,7 +14,7 @@ from vindauga.constants.std_dialog_commands import (cmChangeDir, cdHelpButton, c
                                                     cmDirSelection)
 from vindauga.dialogs.message_box import messageBox
 from vindauga.events.event import Event
-from vindauga.misc.util import getCurDir, fexpand
+from vindauga.utilities.filesystem.path_utils import getCurDir, fexpand
 from vindauga.types.records.data_record import DataRecord
 from vindauga.types.rect import Rect
 from vindauga.widgets.button import Button
@@ -62,16 +63,16 @@ class ChangeDirDialog(Dialog):
             self.setupDialog()
         self.selectNext(False)
 
-    def consumesData(self):
+    def consumesData(self) -> bool:
         return True
 
-    def getData(self):
+    def getData(self) -> DataRecord:
         return self.dirInput.getData()
 
-    def setData(self, data):
+    def setData(self, data: DataRecord) -> None:
         self.dirInput.setData(data.value)
 
-    def handleEvent(self, event):
+    def handleEvent(self, event: Event):
         super().handleEvent(event)
         if event.what == evCommand:
             emc = event.message.command
@@ -130,9 +131,10 @@ class ChangeDirDialog(Dialog):
                 self.dirInput.setData(curDir)
                 self.dirInput.drawView()
 
-    def changeDir(self, path: Union[str, pathlib.Path]) -> bool:
+    @staticmethod
+    def changeDir(path: Union[str, pathlib.Path]) -> bool:
         try:
             os.chdir(path)
             return False
-        except:
+        except Exception:
             return True
