@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
+from __future__ import annotations
 from dataclasses import dataclass
+from typing import Optional
 
 
 @dataclass
@@ -7,4 +9,4 @@ class CrossReferenceNode:
     topic: str
     offset: int
     length: int
-    next: 'CrossReferenceNode'
+    next: Optional[CrossReferenceNode] = None
