@@ -1,14 +1,14 @@
 # -*- coding: utf-8 -*-
 import logging
 from collections import defaultdict
-
+from typing import List
 
 logger = logging.getLogger(__name__)
 
 histories = defaultdict(list)
 
 
-def historyAdd(historyId, historyString):
+def historyAdd(historyId: int, historyString: str):
     if not historyString:
         return
 
@@ -22,19 +22,19 @@ def historyAdd(historyId, historyString):
         history.append(historyString)
 
 
-def historyCount(historyId):
+def historyCount(historyId: int) -> int:
     history = histories[historyId]
     return len(history)
 
 
-def historyStr(historyId, index):
+def historyStr(historyId: int, index: int) -> str:
     history = histories[historyId]
     if index > len(history):
         return ''
     return history[index]
 
 
-def getHistory(historyId):
+def getHistory(historyId: int) -> List[str]:
     return histories[historyId]
 
 
