@@ -1,19 +1,22 @@
 # -*- coding: utf-8 -*-
-import logging
+from __future__ import annotations
 from enum import Enum, auto
+import logging
+from typing import List, Optional, Union, Tuple
 
 from vindauga.constants.command_codes import cmMenu, hcNoContext, cmCommandSetChanged
-from vindauga.constants.event_codes import (evBroadcast, evMouseUp, evMouseMove, evMouseDown, evKeyDown, evCommand,
-                                            evNothing)
-from vindauga.constants.keys import kbEnter, kbEsc, kbNoKey
+import vindauga.constants.event_codes as event_codes
+import vindauga.constants.keys as keys
 from vindauga.events.event import Event
-from vindauga.misc.character_codes import getAltChar
-from vindauga.misc.util import *
-from vindauga.utilities.draw_widget_tree import drawWidgetTree
+from vindauga.utilities.input.character_codes import getAltChar
+from vindauga.utilities.input.key_utils import ctrlToArrow
 from vindauga.types.palette import Palette
 from vindauga.types.point import Point
 from vindauga.types.rect import Rect
 from vindauga.types.view import View
+from .menu import Menu
+
+from .menu_item import MenuItem
 
 logger = logging.getLogger(__name__)
 
@@ -42,19 +45,19 @@ class MenuView(View):
 
     cpMenuView = "\x02\x03\x04\x05\x06\x07"
 
-    def __init__(self, bounds, menu=None, parent=None):
+    def __init__(self, bounds: Rect, menu=None, parent: Optional[MenuView] = None):
         super().__init__(bounds)
         self._parentMenu = parent
         self._current = None
         self.menu = menu
-        self.eventMask |= evBroadcast
+        self.eventMask |= event_codes.evBroadcast
 
     @staticmethod
-    def newSubView(bounds, menu, parentMenu):
+    def newSubView(bounds: Rect, menu, parentMenu) -> 'MenuBox':
         from .menu_box import MenuBox
         return MenuBox(bounds, menu, parentMenu)
 
-    def getItemRect(self, item):
+    def getItemRect(self, item: MenuItem) -> Rect:
         """
         Classes derived from `MenuView` must override this member function in
         order to respond to mouse events. Your overriding functions in derived
@@ -65,7 +68,7 @@ class MenuView(View):
         """
         return Rect(0, 0, 0, 0)
 
-    def execute(self):
+    def execute(self) -> int:
         """
         Executes a menu view until the user selects a menu item or cancels the
         process. Returns the command assigned to the selected menu item, or
@@ -76,7 +79,7 @@ class MenuView(View):
         autoSelect = False
         result = 0
         itemShown = None
-        e = Event(evNothing)
+        e = Event(event_codes.evNothing)
         self._current = self.menu.default
         action = MenuAction.doNothing
 
@@ -84,23 +87,23 @@ class MenuView(View):
             action = MenuAction.doNothing
             self.getEvent(e)
             what = e.what
-            if what == evMouseDown:
+            if what == event_codes.evMouseDown:
                 if self.mouseInView(e.mouse.where) or self.__mouseInOwner(e):
                     self.__trackMouse(e)
                     if self.size.y == 1:
                         autoSelect = True
                 else:
                     action = MenuAction.doReturn
-            elif what == evMouseUp:
+            elif what == event_codes.evMouseUp:
                 action = self.__handleMouseUp(action, e)
-            elif what == evMouseMove:
+            elif what == event_codes.evMouseMove:
                 if e.mouse.buttons:
                     self.__trackMouse(e)
                     if not (self.mouseInView(e.mouse.where) or self.__mouseInOwner(e)) and self.__mouseInMenus(e):
                         action = MenuAction.doReturn
-            elif what == evKeyDown:
+            elif what == event_codes.evKeyDown:
                 action, autoSelect, result = self.__handleEventKeyDown(action, autoSelect, e, result)
-            elif what == evCommand:
+            elif what == event_codes.evCommand:
                 if e.message.command == cmMenu:
                     autoSelect = False
                     if self._parentMenu:
@@ -120,7 +123,7 @@ class MenuView(View):
             else:
                 result = 0
 
-        if e.what != evNothing and (self._parentMenu or e.what == evCommand):
+        if e.what != event_codes.evNothing and (self._parentMenu or e.what == event_codes.evCommand):
             self.putEvent(e)
 
         if self._current:
@@ -130,16 +133,16 @@ class MenuView(View):
 
         return result
 
-    def findItem(self, ch):
+    def findItem(self, ch: Union[str, int]) -> Optional[MenuItem]:
         """
         Returns a pointer to the menu item that has code.upper() as its hot key
         (the highlighted character). Returns None if no such menu item is found or
-        if the menu item is disabled. Note that `findItem()` is case insensitive.
+        if the menu item is disabled. Note that `findItem()` is case-insensitive.
 
         :param ch: Hot key character
         :return: `MenuItem` object
         """
-        if not self.menu.items:
+        if not (self.menu.items and ch):
             return None
 
         if isinstance(ch, int):
@@ -151,7 +154,7 @@ class MenuView(View):
                 return m
         return None
 
-    def getHelpCtx(self):
+    def getHelpCtx(self) -> int:
         """
         By default, this member function returns the help context of the
         current menu selection. If this is `hcNoContext`, the parent menu's
@@ -169,11 +172,11 @@ class MenuView(View):
             return c._current.helpCtx
         return hcNoContext
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         palette = Palette(self.cpMenuView)
         return palette
 
-    def updateMenu(self, menu):
+    def updateMenu(self, menu: Menu) -> bool:
         res = False
         if menu:
             items = (p for p in menu.items if p.name)
@@ -188,41 +191,41 @@ class MenuView(View):
                         res = True
         return res
 
-    def executeMenu(self, event):
+    def executeMenu(self, event: Event):
         self.putEvent(event)
         emc = self.owner.execView(self)
         if emc and self.commandEnabled(emc):
-            event.what = evCommand
+            event.what = event_codes.evCommand
             event.message.command = emc
             event.message.infoPtr = None
             self.putEvent(event)
         self.clearEvent(event)
 
-    def handleEvent(self, event):
+    def handleEvent(self, event: Event):
         if self.menu:
             what = event.what
-            if what == evMouseDown:
+            if what == event_codes.evMouseDown:
                 self.executeMenu(event)
-            elif what == evKeyDown:
+            elif what == event_codes.evKeyDown:
                 if self.findItem(getAltChar(event.keyDown.keyCode)):
                     self.executeMenu(event)
                 else:
                     p = self.hotKey(event.keyDown.keyCode)
                     if p and self.commandEnabled(p.command):
-                        event.what = evCommand
+                        event.what = event_codes.evCommand
                         event.message.command = p.command
                         event.message.infoPtr = None
                         self.putEvent(event)
                         self.clearEvent(event)
-            elif what == evCommand:
+            elif what == event_codes.evCommand:
                 if event.message.command == cmMenu:
                     self.executeMenu(event)
-            elif what == evBroadcast:
+            elif what == event_codes.evBroadcast:
                 if event.message.command == cmCommandSetChanged:
                     if self.updateMenu(self.menu):
                         self.drawView()
 
-    def findHotKey(self, items, keyCode):
+    def findHotKey(self, items: List[MenuItem], keyCode) -> Optional[MenuItem]:
         if items:
             items = (p for p in items if p.name)
             for p in items:
@@ -230,11 +233,11 @@ class MenuView(View):
                     T = self.findHotKey(p.subMenu.items, keyCode)
                     if T:
                         return T
-                elif (not p.disabled) and p.keyCode != kbNoKey and p.keyCode == keyCode:
+                elif (not p.disabled) and p.keyCode != keys.kbNoKey and p.keyCode == keyCode:
                     return p
         return None
 
-    def hotKey(self, keyCode):
+    def hotKey(self, keyCode) -> MenuItem:
         """
         Returns a pointer to the menu item associated with the hot key given
         by `keyCode`. Returns None if no such menu item exists, or if the item is
@@ -246,7 +249,7 @@ class MenuView(View):
         """
         return self.findHotKey(self.menu.items, keyCode)
 
-    def __handleMouseUp(self, action, e):
+    def __handleMouseUp(self, action: MenuAction, e: Event) -> MenuAction:
         mouseActive = self.__trackMouse(e)
         if self.__mouseInOwner(e):
             self._current = self.menu.default
@@ -261,30 +264,31 @@ class MenuView(View):
             action = MenuAction.doNothing
         return action
 
-    def __handleEventKeyDown(self, action, autoSelect, e, result):
+    def __handleEventKeyDown(self, action: MenuAction, autoSelect: bool, e: Event, 
+                             result: int) -> Tuple[MenuAction, bool, int]:
         cta = ctrlToArrow(e.keyDown.keyCode)
-        if cta in {kbUp, kbDown}:
+        if cta in {keys.kbUp, keys.kbDown}:
             if self.size.y != 1:
-                self.__trackKey(cta == kbDown)
-            elif e.keyDown.keyCode == kbDown:
+                self.__trackKey(cta == keys.kbDown)
+            elif e.keyDown.keyCode == keys.kbDown:
                 autoSelect = True
-        elif cta in {kbLeft, kbRight}:
+        elif cta in {keys.kbLeft, keys.kbRight}:
             if not self._parentMenu:
-                self.__trackKey(cta == kbRight)
-            elif cta == kbRight and not self._current.command:
+                self.__trackKey(cta == keys.kbRight)
+            elif cta == keys.kbRight and not self._current.command:
                 action = MenuAction.doSelect
             else:
                 action = MenuAction.doReturn
-        elif cta in {kbHome, kbEnd}:
+        elif cta in {keys.kbHome, keys.kbEnd}:
             if self.size.y != 1:
                 self._current = self.menu.items
-                if e.keyDown.keyCode == kbEnd:
+                if e.keyDown.keyCode == keys.kbEnd:
                     self.__trackKey(False)
-        elif cta == kbEnter:
+        elif cta == keys.kbEnter:
             if self.size.y == 1:
                 autoSelect = True
             action = MenuAction.doSelect
-        elif cta == kbEsc:
+        elif cta == keys.kbEsc:
             action = MenuAction.doReturn
             if not self._parentMenu or (self._parentMenu.size.y != 1):
                 self.clearEvent(e)
@@ -312,11 +316,11 @@ class MenuView(View):
                 action = MenuAction.doReturn
         return action, autoSelect, result
 
-    def __handleMenuAction(self, action, autoSelect, e, result):
+    def __handleMenuAction(self, action: MenuAction, autoSelect: bool, e: Event, result: int) -> int:
         if ((action == MenuAction.doSelect or (action == MenuAction.doNothing and autoSelect))
                 and (self._current and self._current.name)):
             if not self._current.command:
-                if e.what & (evMouseDown | evMouseMove):
+                if e.what & (event_codes.evMouseDown | event_codes.evMouseMove):
                     self.putEvent(e)
 
                 r = self.getItemRect(self._current)
@@ -334,7 +338,7 @@ class MenuView(View):
                 result = self._current.command
         return result
 
-    def __trackMouse(self, e):
+    def __trackMouse(self, e: Event) -> bool:
         mouse = self.makeLocal(e.mouse.where)
         # Because any bails if the first thing matches... self._current will point to the thing that
         # matches
@@ -366,9 +370,9 @@ class MenuView(View):
         while self._current.next is not p:
             self.__nextItem()
 
-    def __trackKey(self, findNext):
+    def __trackKey(self, findNext: bool):
         if not self._current:
-            return False
+            return
 
         action = self.__nextItem if findNext else self.__prevItem
         done = False
@@ -376,7 +380,7 @@ class MenuView(View):
             action()
             done = bool(self._current.name)
 
-    def __mouseInOwner(self, e):
+    def __mouseInOwner(self, e: Event) -> bool:
         if not self._parentMenu or self._parentMenu.size.y != 1:
             return False
 
@@ -384,13 +388,13 @@ class MenuView(View):
         r = self._parentMenu.getItemRect(self._parentMenu._current)
         return mouse in r
 
-    def __mouseInMenus(self, e):
+    def __mouseInMenus(self, e: Event) -> bool:
         p = self._parentMenu
         while p and not p.mouseInView(e.mouse.where):
             p = p.parentMenu
         return p is not None
 
-    def __topMenu(self):
+    def __topMenu(self) -> MenuView:
         p = self
         while p._parentMenu:
             p = p._parentMenu
