@@ -1,13 +1,53 @@
 # -*- coding: utf-8 -*-
+import functools
+
+import atexit
+from copy import copy
 import curses
-from dataclasses import dataclass
+from functools import lru_cache
+import itertools
 import logging
 import os
 from os import kill, execvpe, waitpid
 import platform
 import select
+from signal import *
 import struct
-import signal
+from typing import Tuple, Union
+
+from vindauga.utilities.text.text import Text
+from vindauga.utilities.screen.screen_cell import ScreenCell
+from vindauga.utilities.colours.colour_attribute import ColourAttribute
+from vindauga.utilities.platform.events.utf8_handler import UTF8CharacterAssembler
+
+
+TERMINAL_KEY_TRANSLATION = {
+    '\n': b'\r',
+    curses.KEY_UP: b'\x1B[A',
+    curses.KEY_DOWN: b'\x1B[B',
+    curses.KEY_RIGHT: b'\x1B[C',
+    curses.KEY_LEFT: b'\x1B[D',
+    curses.KEY_BACKSPACE: b'\x1B\b',
+    curses.KEY_IC: b'\x1b[2~',
+    curses.KEY_DC: b'\x1b[3~',
+    curses.KEY_HOME: b'\x1b[7~',
+    curses.KEY_END: b'\x1b[8~',
+    curses.KEY_PPAGE: b'\x1b[5~',
+    curses.KEY_NPAGE: b'\x1b[6~',
+    curses.KEY_SUSPEND: b'\x1A',
+    curses.KEY_F1: b'\x1b[11~',
+    curses.KEY_F2: b'\x1b[12~',
+    curses.KEY_F3: b'\x1b[13~',
+    curses.KEY_F4: b'\x1b[14~',
+    curses.KEY_F5: b'\x1b[15~',
+    curses.KEY_F6: b'\x1b[17~',
+    curses.KEY_F7: b'\x1b[18~',
+    curses.KEY_F8: b'\x1b[19~',
+    curses.KEY_F9: b'\x1b[20~',
+    curses.KEY_F10: b'\x1b[21~',
+    curses.KEY_F11: b'\x1b[23~',
+    curses.KEY_F12: b'\x1b[24~',
+}
 
 PLATFORM_WINDOWS = platform.platform().lower().startswith('windows')
 if not PLATFORM_WINDOWS:
@@ -15,25 +55,13 @@ if not PLATFORM_WINDOWS:
     import pwd
     import termios
     import pty
+    signals = [SIGINT, SIGQUIT]
 else:
     from .windows_shell import WindowsShell
+    signals = [SIGINT, SIGBREAK]
 
 logger = logging.getLogger(__name__)
 
-
-@dataclass
-class Texel:
-    char: str = ''
-    attr: int = 0
-    color: int = 0
-
-
-@dataclass
-class Color:
-    fg: int
-    bg: int
-
-
 defaultFg = 7
 defaultBg = 0
 
@@ -53,9 +81,10 @@ STATE_TITLE_CHANGED = (1 << 8)
 
 class Terminal:
 
-    def __init__(self, width, height, flags, command=None, *commandArgs):
+    def __init__(self, width: int, height: int, flags: int, command=None, shell_type='pwsh', *commandArgs):
         self.rows = height
         self.cols = width
+        self.shell_type = shell_type
         self.col = []
         self.currRow = 0
         self.currCol = 0
@@ -64,19 +93,19 @@ class Terminal:
         self.savedY = 0
         self.scrollMin = 0
         self.scrollMax = height - 1
+        self._utf8_assembler = UTF8CharacterAssembler()  # Handles UTF-8 character assembly
         self.flags = flags
         self.state = 0
         self.fg = defaultFg
         self.bg = defaultBg
         self.__childPid = -1
         self.__ptyFd = None
-        self.colors = defaultFg | defaultBg
         if command:
             self.command = command
             self.commandArgs = [os.path.basename(command)]
             self.commandArgs.extend(commandArgs)
         else:
-            self.command, self.commandArgs = self.getShell()
+            self.command, self.commandArgs = self.getShell(self.shell_type)
         logger.info('Command is %s, %s', self.command, self.commandArgs)
         self.csiParam = []
         self.escBuf = ''
@@ -84,13 +113,53 @@ class Terminal:
             0, 4, 2, 6, 1, 5, 3, 7
             ]
         self.initColors()
+        self._color_cache = {}
+        # Initialize colors after initColors() is called
+        self.colors = self._create_colour_attribute(defaultFg, defaultBg)
         self.title = ''
-        self.cells = [[Texel() for _x in range(width)] for _y in range(height)]
-        self.erase()
+        # Initialize cells with ScreenCell objects containing space characters
+        self.cells = []
+        for _y in range(height):
+            row = []
+            for _x in range(width):
+                cell = ScreenCell()
+                cell.char = ' '  # Use string instead of int
+                cell.attr = self._create_colour_attribute()  # Use proper ColourAttribute
+                row.append(cell)
+            self.cells.append(row)
         self.executeCommand()
+        atexit.register(self._destroy, self.__childPid)
+
+        from .terminal_view import TerminalView
+        if not TerminalView.OriginalSignals:
+            TerminalView.OriginalSignals = {s: signal(s, SIG_IGN) for s in signals}
+
+    def _create_colour_attribute(self, fg: int = None, bg: int = None, attr: int = 0) -> ColourAttribute:
+        """
+        Convert terminal colors to ColourAttribute
+        """
+        # Use current colors if not specified
+        if fg is None:
+            fg = self.fg if hasattr(self, 'fg') else defaultFg
+        if bg is None:
+            bg = self.bg if hasattr(self, 'bg') else defaultBg
+            
+        # Create ColourAttribute from terminal color values
+        # Terminal uses standard ANSI colors (0-15), map to BIOS-style attributes
+        bios_attr = (bg << 4) | fg
+        
+        # Add terminal attributes like reverse, bold, etc.
+        if attr & curses.A_REVERSE:
+            bios_attr = ((bios_attr & 0x0F) << 4) | ((bios_attr & 0xF0) >> 4)
+        if attr & curses.A_BOLD:
+            bios_attr |= 0x08  # High intensity
+
+        if bios_attr not in self._color_cache:
+            self._color_cache[bios_attr] = ColourAttribute.from_bios(bios_attr)
+        return self._color_cache[bios_attr]
 
     @staticmethod
-    def getShell():
+    def getShell(shell_type: str = 'pwsh') -> Tuple[str, list]:
         if not PLATFORM_WINDOWS:
             profile = pwd.getpwuid(os.getuid())
             if not profile:
@@ -100,7 +169,17 @@ class Terminal:
             else:
                 shell = profile.pw_shell
             return shell, [os.path.basename(shell), '-l']
-        return 'cmd.exe', ['/u', '/f:on']
+
+        # Windows shell options
+        if shell_type.lower() == 'cmd':
+            # cmd.exe with UTF-8 encoding
+            return 'cmd.exe', ['/c', 'chcp 65001 >nul: & cmd.exe /E:ON /F:ON']
+        elif shell_type.lower() in ('pwsh', 'powershell'):
+            # PowerShell Core (pwsh) - interactive mode with completion enabled
+            return 'pwsh', ['-NoLogo']
+        else:
+            # Default to PowerShell Core
+            return 'pwsh', ['-NoLogo']
 
     if not PLATFORM_WINDOWS:
         def executeCommand(self):
@@ -113,8 +192,11 @@ class Terminal:
             if not self.__childPid:
                 # Child
                 env = os.environ.copy()
-                signal.signal(signal.SIGINT, signal.SIG_DFL)
                 env['TERM'] = 'rxvt'
+                env['COLUMNS'] = str(self.cols)
+                env['LINES'] = str(self.rows)
+                env["LANG"] = 'en_US.UTF-8'
+                env["LC_CTYPE"] = 'en_US.UTF-8'
                 logger.info('Executing %s, %s', self.command, self.commandArgs)
                 execvpe(self.command, self.commandArgs, env=env)
 
@@ -122,52 +204,64 @@ class Terminal:
                 raise RuntimeError
     else:
         def executeCommand(self):
-            command = [self.command] + self.commandArgs[1:]
+            command = [self.command] + self.commandArgs
             self.__childPid = WindowsShell(command)
             self.__ptyFd = self.__childPid()
-            logger.info('handles: %s', self.__ptyFd)
             if self.__ptyFd is None:
+                logger.error("*** __ptyFd is None, raising RuntimeError")
                 raise RuntimeError
 
-    def destroy(self):
+    @staticmethod
+    def _destroy(pid):
         try:
-            kill(self.__childPid, signal.SIGKILL)
+            kill(pid, SIGKILL)
         except:
             pass
 
+    def destroy(self):
+        try:
+            kill(self.__childPid, SIGKILL)
+        except:
+            pass
+        from .terminal_view import TerminalView
+        if not TerminalView.ActiveTerminals:
+            # Restore original signal handlers, we were the last terminal
+            for signal_, handler in TerminalView.OriginalSignals.items():
+                signal(signal_, handler)
+
     def initColors(self):
-        self.col.append(Color(0, 0))
-        for i in range(8):
-            for j in range(8):
-                if i != 7 or j != 0:
-                    self.col.append(Color(fg=i, bg=j))
+        """
+        Initialize color palette with ColourAttribute objects
+        """
+        self.col.append(ColourAttribute.from_bios(0))  # Default: black on black
+        for bg, fg in itertools.product(range(8), range(8)):
+            if bg != 7 or fg != 0:  # Skip white on black (default is reversed)
+                bios_attr = (bg << 4) | fg
+                self.col.append(ColourAttribute.from_bios(bios_attr))
 
     @property
-    def pid(self):
+    def pid(self) -> int:
         return self.__childPid
 
     @property
-    def ptyFd(self):
+    def ptyFd(self) -> int:
         return self.__ptyFd
 
-    def setColors(self, _fg, _bg):
-        self.colors = defaultFg | (defaultBg << 4)
-        return 0
+    def setColors(self, fg: int, bg: int):
+        """
+        Set current foreground and background colors
+        """
+        self.fg = fg
+        self.bg = bg
+        self.colors = self._create_colour_attribute(fg, bg)
 
-    def getColors(self):
+    def getColors(self) -> ColourAttribute:
+        """
+        Get current colors as ColourAttribute
+        """
         return self.colors
 
-    def pairContentVision(self, c):
-        return self.col[c].fg, self.col[c].bg
-
-    def findColorPair(self, fg, bg):
-        for i in range(64):
-            if self.col[i].fg == fg and self.col[i].bg == bg:
-                return i
-        return 0
-
     def startCSI(self):
-        # logger.info('startCSI() -> %s', self.escBuf)
         verb = self.escBuf[-1]
         self.csiParam = []
 
@@ -178,7 +272,7 @@ class Terminal:
                 continue
 
             if p == ';':
-                if len(self.csiParam) > MAX_CSI_ESC_PARAMS:
+                if len(self.csiParam) >= MAX_CSI_ESC_PARAMS:
                     return
                 self.csiParam.append(0)
             else:
@@ -239,7 +333,7 @@ class Terminal:
 
     def do_CUP(self):
         """
-        Cursor Up
+        Move Cursor
         """
         if not self.csiParam:
             self.currRow = 0
@@ -251,9 +345,11 @@ class Terminal:
         self.currCol = self.csiParam[1] - 1
         self.clampCursorToBounds()
 
-    def do_CUx(self, verb):
+    def do_CUx(self, verb: str):
         """
         Relative Cursor
+        CUU, CUD, CUF, CUB, CNL, CPL, CHA, HPT, VPA, VPR, HPA
+
         :param verb: Cursor command
         """
         n = self.getNumber()
@@ -275,7 +371,7 @@ class Terminal:
         elif verb == 'F':
             self.currRow -= n
             self.currCol = 0
-        elif verb == 'G`':
+        elif verb in 'G`':
             self.currCol = self.csiParam[0] - 1
         elif verb == 'd':
             self.currRow = self.csiParam[0] - 1
@@ -285,16 +381,15 @@ class Terminal:
         """
         Delete Chars
         """
-        # logger.info('do_DCH()')
         n = self.getNumber()
-
         columns = self.cells[self.currRow]
-        end = max(self.cols, self.currCol + n)
-
-        columns[self.currCol: end] = columns[self.currCol + n: self.currCol + n + n]
-        for i in range(end, self.cols):
-            columns[i] = Texel()
-            self._resetCell(columns[i])
+        for i in range(self.currCol, self.cols):
+            if i + n < self.cols:
+                columns[i] = columns[i + n]
+            else:
+                cell = columns[i]
+                cell.char = ' '
+                cell.attr = self._create_colour_attribute()
 
     def do_DECSTBM(self):
         """
@@ -327,24 +422,17 @@ class Terminal:
         if not self.scrollMin and self.scrollMax == self.rows - 1:
             self.state &= ~STATE_SCROLL_SHORT
 
-    def _resetCell(self, cell):
-        cell.char = 0x20
-        cell.attr = self.currAttr
-        cell.color = self.colors
-
     def do_DL(self):
         """
         Delete Lines
         """
         n = self.getNumber()
 
-        for i in range(self.currRow, self.scrollMax):
-            if i + n < self.scrollMax:
+        for i in range(self.currRow, self.scrollMax + 1):
+            if i + n <= self.scrollMax:
                 self.cells[i] = self.cells[i + n]
             else:
-                columns = self.cells[i]
-                for j in range(self.cols):
-                    self._resetCell(columns[j])
+                self._resetRow(i)
 
     def do_ECH(self):
         """
@@ -353,9 +441,7 @@ class Terminal:
         n = self.getNumber()
 
         columns = self.cells[self.currRow]
-        for i in range(self.currCol, self.currCol + n):
-            if i > self.cols:
-                break
+        for i in range(self.currCol, max(self.cols, self.currCol + n)):
             self._resetCell(columns[i])
 
     def do_ED(self):
@@ -386,7 +472,6 @@ class Terminal:
         """
         Erase Line
         """
-        # logger.info('do_EL() -> %s', self.csiParam)
         cmd = 0
         if self.csiParam:
             cmd = self.csiParam[0]
@@ -401,43 +486,32 @@ class Terminal:
             eraseStart = self.currCol
             eraseEnd = self.cols - 1
 
-        # logger.info('Erase %s - %s', eraseStart, eraseEnd)
-
         columns = self.cells[self.currRow]
         for i in range(eraseStart, eraseEnd + 1):
             self._resetCell(columns[i])
 
     def do_ICH(self):
         """
-        Insert Chars
+        Insert Blank Chars
         """
         n = self.getNumber()
 
         columns = self.cells[self.currRow]
-        columns[self.cols: self.cols + n] = columns[self.cols - n: self.cols]
-        # for i in range(self.cols, self.cols + n):
-        #     columns[i] = columns[i - n]
+        for i in range(self.cols - 1, self.currCol + n - 1, -1):
+            columns[i] = columns[i - n]
+
         for i in range(self.currCol, self.currCol + n):
             self._resetCell(columns[i])
 
-    def getNumber(self):
-        n = 1
-        if self.csiParam and self.csiParam[0] > 0:
-            n = self.csiParam[0]
-        return n
-
     def do_IL(self):
+        """
+        Insert Line
+        """
         n = self.getNumber()
-
-        self.cells[self.currRow: self.scrollMax] = self.cells[self.currRow - n: self.scrollMax - n]
-        # for i in range(self.scrollMax, self.currRow - 1, -1):
-        #     self.cells[i] = self.cells[i - n]
-        for i in range(self.currRow, self.currRow + n):
-            if i > self.scrollMax:
-                break
-            columns = self.cells[i]
-            for j in range(self.cols):
-                self._resetCell(columns[j])
+        for i in range(self.scrollMax, self.currRow + n - 1, -1):
+            self.cells[i] = copy(self.cells[i - n])
+        for i in range(self.currRow, max(self.scrollMax, self.currRow + n)):
+            self._resetRow(i)
 
     def do_RESTORECUR(self):
         """
@@ -457,7 +531,6 @@ class Terminal:
         """
         Set Graphics
         """
-        # logger.info('do_SGR() -> %s', self.csiParam)
         if not self.csiParam:
             # Reset
             self.currAttr = curses.A_NORMAL
@@ -476,7 +549,7 @@ class Terminal:
                 continue
 
             if param in {1, 2, 4}:
-                # Bold
+                # 1=bold, 4=underline
                 self.currAttr |= curses.A_BOLD
                 self.colors |= 8
                 continue
@@ -511,7 +584,7 @@ class Terminal:
                 continue
 
             if param in {22, 24}:
-                # Bold Off
+                # Bold/Underline Off
                 self.currAttr &= ~curses.A_BOLD
                 self.colors &= ~8
                 continue
@@ -533,23 +606,24 @@ class Terminal:
             elif 40 <= param <= 47:
                 # set bg color
                 self.bg = self.colorValues[param - 40] & 0x7
+            elif 90 <= param <= 97:
+                self.fg = self.colorValues[param - 90] & 0xF
+                self.currAttr |= curses.A_BOLD
+            elif 100 <= param <= 107:
+                self.bg = self.colorValues[param - 100] & 0x7
+                self.currAttr |= curses.A_BOLD
             elif param == 39:
                 # Reset fg
                 self.fg = defaultFg
             elif param == 49:
                 self.bg = defaultBg
 
-            self.colors = self.fg | (self.bg << 4)
-            if self.currAttr & curses.A_BOLD:
-                self.colors |= 8
-            if self.currAttr & curses.A_BLINK:
-                self.colors |= 128
+            self.colors = self._create_colour_attribute(self.fg, self.bg, self.currAttr)
 
     def do_DEC_RM(self):
         """
         Reset Mode
         """
-        # logger.info('do_DEC_RM() -> %s', self.csiParam)
         if not self.csiParam:
             return
         for param in self.csiParam:
@@ -562,7 +636,6 @@ class Terminal:
         """
         Set Mode
         """
-        # logger.info('do_DEC_SM() %s', self.csiParam)
         if not self.csiParam:
             return
 
@@ -572,34 +645,37 @@ class Terminal:
             elif param == 9:
                 self.state |= STATE_MOUSE
 
-    def erase(self):
-        for row in self.cells:
-            for cell in row:
-                self._resetCell(cell)
+    def getNumber(self) -> int:
+        n = 1
+        if self.csiParam and self.csiParam[0] > 0:
+            n = self.csiParam[0]
+        return n
 
-    def eraseRow(self, rowNum):
+    def erase(self):
+        for i in range(self.rows):
+            self._resetRow(i)
+
+    def eraseRow(self, rowNum: int):
         if rowNum == -1:
             rowNum = self.currRow
 
-        columns = self.cells[rowNum]
-        for i in range(self.cols):
-            self._resetCell(columns[i])
+        self._resetRow(rowNum)
 
-    def eraseRows(self, startRow):
+    def eraseRows(self, startRow: int):
         if startRow < 0:
             return
 
         for row in range(startRow, self.rows):
             self.eraseRow(row)
 
-    def eraseCol(self, col):
-        if col == -1:
+    def eraseCol(self, col: int):
+        if col < 0:
             col = self.currCol
 
         for i in range(self.rows):
             self._resetCell(self.cells[i][col])
 
-    def eraseCols(self, startCol):
+    def eraseCols(self, startCol: int):
         if startCol < 0:
             return
 
@@ -615,15 +691,11 @@ class Terminal:
         self.escBuf = ''
 
     def setTitle(self):
-        logger.info('setTitle() -> %s', self.escBuf)
         if self.escBuf[1] != '0' or self.escBuf[2] != ';':
             return
         self.title = ''
-        for i in range(3, 79):
-            if self.escBuf[i] == '\a':
-                break
-            self.title += self.escBuf[i]
-
+        title, _ = self.escBuf[3:].split('\a', 1)
+        self.title = title
         self.state |= STATE_TITLE_CHANGED
 
     def tryEscapeSequence(self):
@@ -651,14 +723,18 @@ class Terminal:
         if firstChar == '[' and self.validEscapeSuffix(lastChar):
             self.startCSI()
             self.escapeCancel()
+            return
         elif firstChar == ']' and lastChar == '\a':
             self.setTitle()
             self.escapeCancel()
+            return
         if len(self.escBuf) + 1 >= ESC_SEQ_BUF_SIZE:
             self.escapeCancel()
 
+
     @staticmethod
-    def validEscapeSuffix(c):
+    @lru_cache(maxsize=256)
+    def validEscapeSuffix(c: str) -> bool:
         return (
                 ('a' <= c <= 'z') or
                 ('A' <= c <= 'Z') or
@@ -666,34 +742,52 @@ class Terminal:
         )
 
     def clampCursorToBounds(self):
-        self.currRow = max(min(self.rows, self.currRow), 0)
-        self.currCol = max(min(self.cols, self.currCol), 0)
+        self.currRow = max(min(self.rows - 1, self.currRow), 0)
+        self.currCol = max(min(self.cols - 1, self.currCol), 0)
 
-    def putChar(self, c):
+    def putChar(self, c: str):
+        """
+        Put a UTF-8 character, handling combining characters properly
+        """
         if self.currCol >= self.cols:
             self.currCol = 0
             self.scrollDown()
 
-        columns = self.cells[self.currRow]
-        columns[self.currCol].char = c
-        columns[self.currCol].attr = self.currAttr
-        columns[self.currCol].color = self.colors
-        self.currCol += 1
+        cell = self.cells[self.currRow][self.currCol]
+        
+        # Check if this is a combining character (width = 0)
+        char_width = Text.width(c)
+        if char_width == 0 and self.currCol > 0:
+            # Combining character - add to previous cell
+            prev_cell = self.cells[self.currRow][self.currCol - 1]
+            if prev_cell.char:
+                prev_cell.char += c
+            return  # Don't advance cursor for combining characters
+        
+        # Normal character
+        cell.char = c
+        cell.attr = self._create_colour_attribute(attr=self.currAttr)
+        
+        # Advance cursor by character width (1 for normal chars, 2 for wide chars)
+        self.currCol += max(1, char_width)
 
-    def renderCtrlChar(self, c):
-        c = chr(c)
+    def renderCtrlChar(self, c: str):
         if c == '\r':
+            # carriage return
             self.currCol = 0
         elif c == '\n':
+            # line feed
             self.scrollDown()
         elif c == '\b':
+            # backspace
             if self.currCol > 0:
                 self.currCol -= 1
         elif c == '\t':
-            self.putChar(0x20)
+            # tab
+            if not self.currCol % 8:
+                self.putChar(' ')
             while self.currCol % 8:
-                self.putChar(0x20)
-
+                self.putChar(' ')
         elif c == '\x1B':
             self.escapeStart()
         elif c == '\x0E':
@@ -715,27 +809,45 @@ class Terminal:
             # NFI
             pass
 
-    def isModeACS(self):
-        return self.state & STATE_ALT_CHARSET
+    def isModeACS(self) -> bool:
+        return bool(self.state & STATE_ALT_CHARSET)
 
-    def isModeEscaped(self):
-        return self.state & STATE_ESCAPE_MODE
+    def isModeEscaped(self) -> bool:
+        return bool(self.state & STATE_ESCAPE_MODE)
 
-    def render(self, data):
-        for c in data:
-            c = ord(c)
-            if c == 0:
+    def render(self, data: str):
+        """
+        Render UTF-8 text properly, handling combining characters and escape sequences
+        """
+        text_pos = 0
+        
+        while text_pos < len(data):
+            # Get next character with proper UTF-8 handling
+            success, new_text_pos, char_width = Text.next_with_width(data, text_pos)
+            if not success:
+                break
+                
+            char = data[text_pos:new_text_pos]
+            
+            # Skip null characters
+            if char == '\x00':
+                text_pos = new_text_pos
                 continue
+                
             if self.isModeEscaped() and len(self.escBuf) < ESC_SEQ_BUF_SIZE:
-                self.escBuf += chr(c)
+                self.escBuf += char
                 self.tryEscapeSequence()
             else:
-                if 1 <= c <= 31:
-                    self.renderCtrlChar(c)
-                    continue
-                self.putChar(c)
+                # Check if it's a control character
+                if len(char) == 1 and 1 <= ord(char) <= 31:
+                    self.renderCtrlChar(char)
+                else:
+                    # Normal character (including combining characters)
+                    self.putChar(char)
+            
+            text_pos = new_text_pos
 
-    def resize(self, width, height):
+    def resize(self, width: int, height: int):
         if not (width and height):
             return
 
@@ -744,13 +856,32 @@ class Terminal:
         startX = self.cols
         startY = self.rows
 
-        cells = self.cells
-        self.cells = [[Texel() for _x in range(width)] for _y in range(height)]
+        if deltaY < 0:
+            cells = self.cells[-deltaY:]
+            self.currRow += deltaY
+        else:
+            cells = self.cells[:]
+
+        # Initialize cells with ScreenCell objects
+        self.cells = []
+        for _y in range(height):
+            row = []
+            for _x in range(width):
+                cell = ScreenCell()
+                cell.char = ' '
+                cell.attr = self._create_colour_attribute()
+                row.append(cell)
+            self.cells.append(row)
+
         for i, c in enumerate(cells):
             if i < len(self.cells):
                 self.cells[i][:width] = c[:width]
-            if deltaX > 0:
-                self.cells[i].extend(Texel() for _ in range(deltaX))
+                if deltaX > 0:
+                    for _ in range(deltaX):
+                        cell = ScreenCell()
+                        cell.char = ' '
+                        cell.attr = self._create_colour_attribute()
+                        self.cells[i].append(cell)
 
         self.cols = width
         self.rows = height
@@ -763,13 +894,17 @@ class Terminal:
             self.eraseCols(startX)
         if deltaY > 0:
             self.eraseRows(startY)
-        self.currCol = 0
-        self.currRow = 0
+
+        if self.currRow >= self.rows:
+            self.currRow = 0
+
+        if self.currCol >= self.cols:
+            self.currCol = 0
 
         if not PLATFORM_WINDOWS:
             b = struct.pack('HHHH', height, width, 0, 0)
             fcntl.ioctl(self.__ptyFd, termios.TIOCSWINSZ, b)
-            kill(self.__childPid, signal.SIGWINCH)
+            kill(self.__childPid, SIGWINCH)
 
     def scrollDown(self):
         self.currRow += 1
@@ -777,9 +912,15 @@ class Terminal:
             return
 
         self.currRow = self.scrollMax
-        self.cells[self.scrollMin:self.scrollMax] = self.cells[self.scrollMin + 1: self.scrollMax + 1]
-        self.cells[self.scrollMax] = [Texel() for _ in range(self.cols)]
-        self.eraseRow(self.scrollMax)
+        self.cells[self.scrollMin: self.scrollMax] = self.cells[self.scrollMin + 1: self.scrollMax + 1]
+        # Create new row of ScreenCells
+        new_row = []
+        for _ in range(self.cols):
+            cell = ScreenCell()
+            cell.char = ' '
+            cell.attr = self._create_colour_attribute()
+            new_row.append(cell)
+        self.cells[self.scrollMax] = new_row
 
     def scrollUp(self):
         self.currRow -= 1
@@ -788,45 +929,23 @@ class Terminal:
 
         self.currRow = self.scrollMin
         self.cells[self.scrollMin + 1:self.scrollMax] = self.cells[self.scrollMin:self.scrollMax - 1]
-        self.cells[self.scrollMin] = [Texel() for _ in range(self.cols)]
-        self.eraseRow(self.scrollMin)
+        # Create new row of ScreenCells
+        new_row = []
+        for _ in range(self.cols):
+            cell = ScreenCell()
+            cell.char = ' '
+            cell.attr = self._create_colour_attribute()
+            new_row.append(cell)
+        self.cells[self.scrollMin] = new_row
 
     def write(self, keyCode):
-        keyTrans = {
-            '\n': b'\r',
-            curses.KEY_UP: b'\x1B[A',
-            curses.KEY_DOWN: b'\x1B[B',
-            curses.KEY_RIGHT: b'\x1B[C',
-            curses.KEY_LEFT: b'\x1B[D',
-            curses.KEY_BACKSPACE: b'\x1B\b',
-            curses.KEY_IC: b'\x1b[2~',
-            curses.KEY_DC: b'\x1b[3~',
-            curses.KEY_HOME: b'\x1b[7~',
-            curses.KEY_END: b'\x1b[8~',
-            curses.KEY_PPAGE: b'\x1b[5~',
-            curses.KEY_NPAGE: b'\x1b[6~',
-            curses.KEY_SUSPEND: b'\x1A',
-            curses.KEY_F1: b'\x1b[11~',
-            curses.KEY_F2: b'\x1b[12~',
-            curses.KEY_F3: b'\x1b[13~',
-            curses.KEY_F4: b'\x1b[14~',
-            curses.KEY_F5: b'\x1b[15~',
-            curses.KEY_F6: b'\x1b[17~',
-            curses.KEY_F7: b'\x1b[18~',
-            curses.KEY_F8: b'\x1b[19~',
-            curses.KEY_F9: b'\x1b[20~',
-            curses.KEY_F10: b'\x1b[21~',
-            curses.KEY_F11: b'\x1b[23~',
-            curses.KEY_F12: b'\x1b[24~',
-        }
-        buffer = keyTrans.get(keyCode)
-        if not buffer:
-            self.writePipe(keyCode)
-        else:
+        if buffer := TERMINAL_KEY_TRANSLATION.get(keyCode):
             self.writePipe(buffer)
+        else:
+            self.writePipe(keyCode)
 
     if not PLATFORM_WINDOWS:
-        def readPipe(self):
+        def readPipe(self) -> int:
             if self.__ptyFd < 0:
                 return -1
 
@@ -844,36 +963,53 @@ class Terminal:
             poller = select.poll()
             poller.register(self.__ptyFd, select.POLLIN)
             count = 0
-            while poller.poll(10):
+            while poller.poll(5):
                 try:
-                    buffer = os.read(self.__ptyFd, 16384).decode('utf-8')
+                    # Read raw bytes and let Text class handle UTF-8 decoding
+                    buffer = os.read(self.__ptyFd, 16380)
                     if buffer:
-                        try:
-                            self.render(buffer)
-                        except:
-                            logger.exception('terminal.render()')
-                        count += len(buffer)
+                        # Process each byte through UTF8 assembler
+                        decoded_text = ""
+                        for byte_val in buffer:
+                            char = self._utf8_assembler.add_byte(byte_val)
+                            if char is not None:
+                                decoded_text += char
+
+                        if decoded_text:
+                            try:
+                                self.render(decoded_text)
+                            except:
+                                logger.exception('terminal.render()')
+                            count += len(decoded_text)
                     else:
                         break
                 except:
                     break
-                break
             return count
     else:
-        def readPipe(self):
+        def readPipe(self) -> int:
             count = 0
             try:
                 buffer = self.__childPid.read()
             except BrokenPipeError:
+                self.state |= STATE_CHILD_EXITED
+                logger.exception('readPipe() - child exited?')
                 return -1
 
             if buffer:
-                buffer = buffer.decode('utf-8')
-                count = len(buffer)
-                try:
-                    self.render(buffer)
-                except:
-                    logger.exception('terminal.render()')
+                # Process each byte through UTF8 assembler
+                decoded_text = ""
+                for byte_val in buffer:
+                    char = self._utf8_assembler.add_byte(byte_val)
+                    if char is not None:
+                        decoded_text += char
+
+                if decoded_text:
+                    count = len(decoded_text)
+                    try:
+                        self.render(decoded_text)
+                    except:
+                        logger.exception('terminal.render()')
             return count
 
     if not PLATFORM_WINDOWS:
@@ -891,3 +1027,15 @@ class Terminal:
             else:
                 self.__childPid.write(keyCode)
 
+    def _resetCell(self, cell: ScreenCell):
+        cell.char = ' '
+        cell.attr = self._create_colour_attribute()
+
+    def _resetRow(self, rowNum: int):
+        new_row = []
+        for _ in range(self.cols):
+            cell = ScreenCell()
+            cell.char = ' '
+            cell.attr = self._create_colour_attribute()
+            new_row.append(cell)
+        self.cells[rowNum] = new_row
