@@ -1,28 +1,89 @@
 # -*- coding: utf-8 -*-
+from __future__ import annotations
 import curses
 import logging
 import os
 
+import wcwidth
+
 from vindauga.constants.command_codes import cmPaste
-from vindauga.constants.event_codes import evMouseUp, evKeyDown, evMouseDown, evCommand
+from vindauga.utilities.text.text import Text
+from vindauga.constants.event_codes import evMouseUp, evKeyDown, evMouseDown, evCommand, evMouseWheel
 from vindauga.constants.option_flags import ofSelectable
 from vindauga.constants.keys import *
 from vindauga.events.event import Event
-from vindauga.misc.clipboard import Clipboard
+from vindauga.utilities.support.clipboard.clipboard import Clipboard
 from vindauga.types.collections.collection import Collection
 from vindauga.types.draw_buffer import DrawBuffer
 from vindauga.types.point import Point
+from vindauga.types.rect import Rect
 from vindauga.types.view import View
+from vindauga.widgets.window import Window
 
 from .terminal import Terminal, STATE_CURSOR_INVIS, STATE_TITLE_CHANGED, STATE_MOUSE
 
+ESC = '\x1B'
+TERMINAL_KEY_MAP = {
+    kbEnter: (10, 0),
+    kbEsc: (ESC, 0),
+    kbDown: (curses.KEY_DOWN, 0),
+    kbUp: (curses.KEY_UP, 0),
+    kbLeft: (curses.KEY_LEFT, 0),
+    kbRight: (curses.KEY_RIGHT, 0),
+    kbBackSpace: (curses.KEY_BACKSPACE, 0),
+    kbTab: (9, 0),
+    kbPgDn: (curses.KEY_NPAGE, 0),
+    kbPgUp: (curses.KEY_PPAGE, 0),
+    kbHome: (curses.KEY_HOME, 0),
+    kbEnd: (curses.KEY_END, 0),
+    kbIns: (curses.KEY_IC, 0),
+    kbDel: (curses.KEY_DC, 0),
+    kbF1: (curses.KEY_F1, 0),
+    kbF2: (curses.KEY_F2, 0),
+    kbF3: (curses.KEY_F3, 0),
+    kbF4: (curses.KEY_F4, 0),
+    kbF5: (curses.KEY_F5, 0),
+    kbF6: (curses.KEY_F6, 0),
+    kbF7: (curses.KEY_F7, 0),
+    kbF8: (curses.KEY_F8, 0),
+    kbF9: (curses.KEY_F9, 0),
+    kbF10: (curses.KEY_F10, 0),
+    kbAltA: (ESC, 'A'),
+    kbAltB: (ESC, 'B'),
+    kbAltC: (ESC, 'C'),
+    kbAltD: (ESC, 'D'),
+    kbAltE: (ESC, 'E'),
+    kbAltF: (ESC, 'F'),
+    kbAltG: (ESC, 'G'),
+    kbAltH: (ESC, 'H'),
+    kbAltI: (ESC, 'I'),
+    kbAltJ: (ESC, 'J'),
+    kbAltK: (ESC, 'K'),
+    kbAltL: (ESC, 'L'),
+    kbAltM: (ESC, 'M'),
+    kbAltN: (ESC, 'N'),
+    kbAltO: (ESC, 'O'),
+    kbAltP: (ESC, 'P'),
+    kbAltQ: (ESC, 'Q'),
+    kbAltR: (ESC, 'R'),
+    kbAltS: (ESC, 'S'),
+    kbAltT: (ESC, 'T'),
+    kbAltU: (ESC, 'U'),
+    kbAltV: (ESC, 'V'),
+    kbAltW: (ESC, 'W'),
+    kbAltX: (ESC, 'X'),
+    kbAltY: (ESC, 'Y'),
+    kbAltZ: (ESC, 'Z'),
+}
+
 logger = logging.getLogger(__name__)
 
 
 class TerminalView(View):
     ActiveTerminals = Collection()
+    OriginalSignals = dict()
 
-    def __init__(self, bounds, parent, command=None, *commandArgs):
+    def __init__(self, bounds: Rect, parent: Window, command=None, *commandArgs):
         super().__init__(bounds)
         self.window = parent
         self.__clipboard = Clipboard()
@@ -35,21 +96,28 @@ class TerminalView(View):
     def draw(self):
         minY = min(self.size.y, self.terminal.rows)
         minX = min(self.size.x, self.terminal.cols)
-
         for y in range(minY):
-            buffer = DrawBuffer(True)
-            for x in range(minX):
+            buffer = DrawBuffer()
+            x = 0
+            right = minX
+            while x < right:
                 cell = self.terminal.cells[y][x]
-                attr = cell.attr
-                c = cell.color
-                if attr & curses.A_REVERSE:
-                    c = self.reverseColor(c)
+                cell_char = cell.char
+                c_w = Text.width(cell_char) if cell_char else 1
+                colour_attr = cell.attr
+                buffer.putChar(x, cell_char, colour_attr)
+                if c_w and c_w > 0:
+                    buffer.putAttribute(x, colour_attr)
+                if c_w and c_w > 1:
+                    right -= (c_w - 1)
+                x += 1
+            self.writeLine(0, y, min(x, minX), 1, buffer)
 
-                buffer.putAttribute(x, c)
-                buffer.putChar(x, chr(cell.char))
-            self.writeLine(0, y, minX, 1, buffer)
-
-        self.setCursor(self.terminal.currCol, self.terminal.currRow)
+        # Set cursor position with bounds checking
+        cursor_x = min(self.terminal.currCol, self.size.x - 1)
+        cursor_y = min(self.terminal.currRow, self.size.y - 1)
+        if cursor_x >= 0 and cursor_y >= 0:
+            self.setCursor(cursor_x, cursor_y)
 
         if self.terminal.state & STATE_CURSOR_INVIS:
             self.hideCursor()
@@ -72,8 +140,10 @@ class TerminalView(View):
                 self.terminal.write(ch[0])
             if ch[1]:
                 self.terminal.write(ch[1])
+            # Immediately check for terminal output and update display
+            self.handleTerminal(self)
             self.clearEvent(event)
-        elif event.what in {evMouseDown, evMouseUp}:
+        elif event.what in {evMouseDown, evMouseUp, evMouseWheel}:
             if self.terminal.state & STATE_MOUSE:
                 self.sendMouseEvent(event)
             if not self.terminal.state & STATE_MOUSE:
@@ -102,7 +172,7 @@ class TerminalView(View):
             b = chr(35)
         bx = chr(local.x + 33)
         by = chr(local.y + 33)
-        buffer = bytes('\033[M{b}{bx}{by}'.format(b=b, bx=bx, by=by))
+        buffer = bytes(f'\033[M{b}{bx}{by}')
         os.write(self.terminal.ptyFd, buffer)
 
     def tryPaste(self, event: Event, clip: int):
@@ -118,72 +188,17 @@ class TerminalView(View):
         self.drawView()
 
     @staticmethod
-    def decodeKey(k):
-        ESC = '\x1B'
-        keyMap = {
-            kbEnter: (10, 0),
-            kbEsc: (ESC, 0),
-            kbDown: (curses.KEY_DOWN, 0),
-            kbUp: (curses.KEY_UP, 0),
-            kbLeft: (curses.KEY_LEFT, 0),
-            kbRight: (curses.KEY_RIGHT, 0),
-            kbBackSpace: (curses.KEY_BACKSPACE, 0),
-            kbTab: (9, 0),
-            kbPgDn: (curses.KEY_NPAGE, 0),
-            kbPgUp: (curses.KEY_PPAGE, 0),
-            kbHome: (curses.KEY_HOME, 0),
-            kbEnd: (curses.KEY_END, 0),
-            kbIns: (curses.KEY_IC, 0),
-            kbDel: (curses.KEY_DC, 0),
-            kbF1: (curses.KEY_F1, 0),
-            kbF2: (curses.KEY_F2, 0),
-            kbF3: (curses.KEY_F3, 0),
-            kbF4: (curses.KEY_F4, 0),
-            kbF5: (curses.KEY_F5, 0),
-            kbF6: (curses.KEY_F6, 0),
-            kbF7: (curses.KEY_F7, 0),
-            kbF8: (curses.KEY_F8, 0),
-            kbF9: (curses.KEY_F9, 0),
-            kbF10: (curses.KEY_F10, 0),
-            kbAltA: (ESC, 'A'),
-            kbAltB: (ESC, 'B'),
-            kbAltC: (ESC, 'C'),
-            kbAltD: (ESC, 'D'),
-            kbAltE: (ESC, 'E'),
-            kbAltF: (ESC, 'F'),
-            kbAltG: (ESC, 'G'),
-            kbAltH: (ESC, 'H'),
-            kbAltI: (ESC, 'I'),
-            kbAltJ: (ESC, 'J'),
-            kbAltK: (ESC, 'K'),
-            kbAltL: (ESC, 'L'),
-            kbAltM: (ESC, 'M'),
-            kbAltN: (ESC, 'N'),
-            kbAltO: (ESC, 'O'),
-            kbAltP: (ESC, 'P'),
-            kbAltQ: (ESC, 'Q'),
-            kbAltR: (ESC, 'R'),
-            kbAltS: (ESC, 'S'),
-            kbAltT: (ESC, 'T'),
-            kbAltU: (ESC, 'U'),
-            kbAltV: (ESC, 'V'),
-            kbAltW: (ESC, 'W'),
-            kbAltX: (ESC, 'X'),
-            kbAltY: (ESC, 'Y'),
-            kbAltZ: (ESC, 'Z'),
-        }
-        if k in keyMap:
-            return keyMap[k]
-        return -1, 0
+    def decodeKey(k: int) -> tuple[int, int]:
+        return TERMINAL_KEY_MAP.get(k, (-1, 0))
 
     @staticmethod
-    def reverseColor(color):
+    def reverseColor(color: int) -> int:
         a = color & 0x0F
         b = color & 0xF0
         return (a << 4) | (b >> 4)
 
     @staticmethod
-    def handleTerminal(window, *args):
+    def handleTerminal(window: TerminalView, *args):
         bytesRead = window.terminal.readPipe()
         if bytesRead > 0:
             window.drawView()
@@ -196,3 +211,7 @@ class TerminalView(View):
         Call me on idle; `TerminalView.updateTerminals()`
         """
         TerminalView.ActiveTerminals.forEach(TerminalView.handleTerminal)
+
+    def destroy(self):
+        logger.info('Cleaning up %s', self)
+        self.terminal.destroy()
