@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
+from dataclasses import dataclass
 from .collection import Collection
 
 
+@dataclass(frozen=True)
 class DirEntry:
-    def __init__(self, txt, directory):
-        self.displayText = txt
-        self.directory = directory
+    displayText: str
+    directory: str
 
     def dir(self):
         return self.directory
@@ -14,7 +15,7 @@ class DirEntry:
         return self.displayText
 
     def __repr__(self):
-        return '<DirEntry: {}>'.format(self.directory)
+        return f'<DirEntry: {self.directory}>'
 
 
 class DirCollection(Collection):
