@@ -19,11 +19,11 @@ class SortedCollection(Collection):
         super().__init__(iterable)
         self.sort()
 
-    def __setitem__(self, index, value):
+    def __setitem__(self, index: int, value):
         super().__setitem__(index, value)
         self.sort()
 
-    def insert(self, index, value):
+    def insert(self, index: int, value):
         super().append(value)
         self.sort()
 
@@ -31,10 +31,10 @@ class SortedCollection(Collection):
         super().append(value)
         self.sort()
 
-    def sort(self, reverse=False):
+    def sort(self, reverse: bool = False):
         super().sort(key=cmp_to_key(self._compare), reverse=reverse)
 
-    def _compare(self, key1, key2):
+    def _compare(self, key1, key2) -> int:
         """
         `_compare()` is a function that must be overridden in all
         derived classes.
