@@ -7,4 +7,6 @@ class Display(IntEnum):
     smCO80 = 0x0003
     smMono = 0x0007
     smFont8x8 = 0x0100
-    smCO256 = 0x200
+    smColor256 = 0x200
+    smColorHigh = 0x400
+    smUpdate = 0x8000
