@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
+from __future__ import annotations
 from dataclasses import dataclass
 from functools import total_ordering
+from typing import Union
 
 
 @total_ordering
@@ -16,41 +18,41 @@ class Point:
         yield self.x
         yield self.y
 
-    def __sub__(self, other):
+    def __sub__(self, other: Point) -> Point:
         x = self.x - other.x
         y = self.y - other.y
         return Point(x, y)
 
-    def __add__(self, other):
+    def __add__(self, other: Point) -> Point:
         x = self.x + other.x
         y = self.y + other.y
         return Point(x, y)
 
-    def __eq__(self, other):
+    def __eq__(self, other: Union[Point, tuple, list]) -> bool:
         if isinstance(other, (Point,)):
             return self.x == other.x and self.y == other.y
         elif isinstance(other, (tuple, list)):
             return self.x == other[0] and self.y == other[1]
 
-    def __ne__(self, other):
+    def __ne__(self, other: Union[Point, tuple, list]) -> bool:
         if isinstance(other, (Point,)):
             return self.x != other.x or self.y != other.y
         elif isinstance(other, (tuple, list)):
             return self.x != other[0] or self.y != other[1]
 
-    def __lt__(self, other):
+    def __lt__(self, other: Point) -> bool:
         # I am closer to 0, 0 than the other
         return abs(self.x) < abs(other.x) or abs(self.y) < abs(other.y)
 
-    def __iadd__(self, other):
+    def __iadd__(self, other: Point) -> Point:
         self.x += other.x
         self.y += other.y
         return self
 
-    def __isub__(self, other):
+    def __isub__(self, other: Point) -> Point:
         self.x -= other.x
         self.y -= other.y
         return self
 
     def __repr__(self):
-        return '<Point({0.x}, {0.y})>'.format(self)
+        return f'<Point({self.x}, {self.y})>'
