@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import logging
 from threading import Timer as _Timer, RLock
+from typing import Optional
 
 logger = logging.getLogger(__name__)
 
@@ -11,11 +12,11 @@ class Timer:
     """
     def __init__(self):
         self.__lock = RLock()
-        self._timer: _Timer = None
+        self._timer: Optional[_Timer] = None
         self._running: bool = False
         self._expired: bool = True
 
-    def start(self, timeout) -> None:
+    def start(self, timeout: float) -> float:
         self.stop()
         with self.__lock:
             self._timer = _Timer(timeout, self.expire)
