@@ -4,7 +4,7 @@ from .validator import Validator
 
 class LookupValidator(Validator):
 
-    def isValid(self, item):
+    def isValid(self, item) -> bool:
         return self.lookup(item)
 
     def lookup(self, item) -> bool:
