@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+from gettext import gettext as _
+
 from vindauga.constants.message_flags import mfError, mfOKButton
 from vindauga.dialogs.message_box import messageBox
 from .lookup_validator import LookupValidator
@@ -13,9 +15,9 @@ class StringLookupValidator(LookupValidator):
         self.strings = collection
 
     def error(self):
-        messageBox(self.errorMsg, mfError, (mfOKButton,))
+        messageBox(self.errorMsg, mfError, [mfOKButton])
 
-    def lookup(self, s: str):
+    def lookup(self, s: str) -> bool:
         return s in self.strings
 
     def newStringList(self, collection):
