@@ -1,6 +1,12 @@
 # -*- coding: utf-8 -*-
+from __future__ import annotations
+import copy
+import itertools
 import logging
 import sys
+from typing import List, Optional, Set, Union
+
+import wcwidth
 
 from vindauga.constants.command_codes import (cmClose, cmNext, cmPrev, cmZoom, cmResize, cmReleasedFocus, cmCancel,
                                               cmReceivedFocus,
@@ -16,15 +22,22 @@ from vindauga.constants.option_flags import ofSelectable, ofTopSelect, ofFirstCl
 from vindauga.constants.state_flags import (sfVisible, sfCursorVis, sfCursorIns, sfShadow, sfSelected,
                                             sfFocused, sfDragging, sfDisabled, sfModal, sfExposed)
 from vindauga.events.event import Event
-from vindauga.misc.message import message
-from vindauga.misc.util import clamp
+from vindauga.utilities.message import message
+from vindauga.utilities.math_utils import clamp
+from vindauga.utilities.colours.colour_attribute import ColourAttribute, reverse_attribute, get_style, set_style, get_back
+from vindauga.utilities.colours.attribute_pair import AttributePair
+from vindauga.utilities.platform.system_interface import systemInterface
+from vindauga.utilities.screen.screen_cell import ScreenCell, set_cell, set_attr
+
 from .command_set import CommandSet
-from .draw_buffer import DrawBuffer, BufferArray, LINE_WIDTH
+from .cursor import Cursor
+from .draw_buffer import DrawBuffer, LINE_WIDTH
 from .palette import Palette
 from .point import Point
 from .rect import Rect
 from .screen import Screen
 from .vindauga_object import VindaugaObject
+from vindauga.utilities.colours.style_mask import StyleMask
 
 logger = logging.getLogger(__name__)
 
@@ -32,7 +45,7 @@ SHADOW_SIZE = Point(2, 1)
 SHADOW_ATTR = 0x08
 
 
-def initCommands():
+def initCommands() -> CommandSet:
     temp = CommandSet()
     for i in range(256):
         if i not in {cmZoom, cmClose, cmResize, cmNext, cmPrev}:
@@ -78,21 +91,17 @@ class View(VindaugaObject):
     """
     name = 'View'
 
-    phFocused = 0
-    phPreProcess = 1
-    phPostProcess = 2
-
     normalSelect = 0
     enterSelect = 1
     leaveSelect = 2
 
     # Static friends
-    curCommandSet = initCommands()
+    curCommandSet: CommandSet = initCommands()
     commandSetChanged = False
     showMarkers = False
-    errorAttr = 0xCF
+    errorAttr = 0x4F  # 0xCF
 
-    MOVE_COMMANDS = {kbLeft: Point(-1, 0),
+    MOVE_COMMANDS: dict[int, Point] = {kbLeft: Point(-1, 0),
                      kbRight: Point(1, 0),
                      kbUp: Point(0, -1),
                      kbDown: Point(0, 1),
@@ -101,7 +110,7 @@ class View(VindaugaObject):
 
     TheTopView = None
 
-    def __init__(self, bounds):
+    def __init__(self, bounds: Rect):
         super().__init__()
         self.options = 0
         self.eventMask = (evMouseDown | evKeyDown | evCommand)
@@ -109,7 +118,7 @@ class View(VindaugaObject):
         self.growMode = 0
         self.dragMode = dmLimitLoY
         self.helpCtx = hcNoContext
-        self.owner = None
+        self.owner: Optional['Group'] = None
         self.origin = Point()
         self.size = Point()
         self.cursor = Point()
@@ -119,17 +128,17 @@ class View(VindaugaObject):
         self.children = []
 
     @property
-    def next(self):
+    def next(self) -> View:
         children = self.owner.children
         return children[(children.index(self) + 1) % len(children)]
 
     @property
-    def prev(self):
+    def prev(self) -> View:
         children = self.owner.children
         return children[(children.index(self) - 1) % len(children)]
 
     @property
-    def TopView(self):
+    def TopView(self) -> View:
         if View.TheTopView:
             return View.TheTopView
         p = self
@@ -138,11 +147,12 @@ class View(VindaugaObject):
         return p
 
     @staticmethod
-    def commandEnabled(command):
-        return command > 255 or command in View.curCommandSet
+    def commandEnabled(command: int) -> bool:
+        # return command > 255 or command in View.curCommandSet
+        return command in View.curCommandSet
 
     @staticmethod
-    def disableCommands(commands):
+    def disableCommands(commands: Union[CommandSet, Set]):
         """
         Disables the commands specified in the `commands` argument. If the
         command set is changed by this call, `commandSetChanged` is set True.
@@ -153,7 +163,7 @@ class View(VindaugaObject):
         View.curCommandSet.disableCmd(commands)
 
     @staticmethod
-    def disableCommand(command):
+    def disableCommand(command: int):
         """
         Disables the given command. If the  command set is changed by the call, `commandSetChanged` is set True.
         :param command: Command to disable
@@ -162,7 +172,7 @@ class View(VindaugaObject):
         View.curCommandSet.disableCmd(command)
 
     @staticmethod
-    def enableCommands(commands):
+    def enableCommands(commands: Union[CommandSet, Set]):
         """
         Enables all the commands in the `commands` argument. If the
         command set is changed by this call, `commandSetChanged` is set True.
@@ -183,7 +193,7 @@ class View(VindaugaObject):
         View.curCommandSet.enableCmd(command)
 
     @staticmethod
-    def getCommands():
+    def getCommands() -> CommandSet:
         """
         Returns the current command set
         :return: `CommandSet` object
@@ -191,7 +201,7 @@ class View(VindaugaObject):
         return View.curCommandSet
 
     @staticmethod
-    def setCommands(commands):
+    def setCommands(commands: Union[CommandSet, Set]):
         """
         Changes the current command set to the given `commands` argument
 
@@ -201,7 +211,7 @@ class View(VindaugaObject):
         View.curCommandSet = commands
 
     @staticmethod
-    def setCmdState(commands, enable):
+    def setCmdState(commands: Union[CommandSet, Set], enable):
         """
         Set or disable commands
 
@@ -214,11 +224,9 @@ class View(VindaugaObject):
             View.disableCommands(commands)
 
     def doRefresh(self):
-        if Screen.lockRefresh:
-            return
         if self.owner and self.owner.lockFlag:
             return
-        Screen.refresh()
+        Screen.screen.refresh()
 
     def shutdown(self):
         self.hide()
@@ -226,7 +234,7 @@ class View(VindaugaObject):
             self.owner.remove(self)
         super().shutdown()
 
-    def sizeLimits(self, minLimit, maxLimit):
+    def sizeLimits(self, minLimit: Point, maxLimit: Point):
         """
         Sets, in the `minLimit` and `maxLimit` arguments, the minimum and maximum values
         that `size` data member may assume.
@@ -245,7 +253,7 @@ class View(VindaugaObject):
         else:
             maxLimit.x = maxLimit.y = sys.maxsize
 
-    def getBounds(self):
+    def getBounds(self) -> Rect:
         """
         Returns the current value of size, the bounding rectangle of the view
         in its owner's coordinate system.
@@ -255,7 +263,7 @@ class View(VindaugaObject):
         os = self.origin + self.size
         return Rect(self.origin.x, self.origin.y, os.x, os.y)
 
-    def getExtent(self):
+    def getExtent(self) -> Rect:
         """
         Returns the extent rectangle of the view with 0, 0 in the topLeft
 
@@ -263,7 +271,7 @@ class View(VindaugaObject):
         """
         return Rect(0, 0, self.size.x, self.size.y)
 
-    def getClipRect(self):
+    def getClipRect(self) -> Rect:
         """
         Returns the clipping rectangle: the smallest rectangle which needs
         to be redrawn in a `draw()` call.
@@ -279,7 +287,7 @@ class View(VindaugaObject):
         clip.move(-self.origin.x, -self.origin.y)
         return clip
 
-    def mouseInView(self, mouse):
+    def mouseInView(self, mouse: Point) -> bool:
         """
         Returns True if the `mouse` argument (given in global coordinates) is
         within the calling view. Call `makeGlobal` and `makeLocal` to
@@ -292,7 +300,7 @@ class View(VindaugaObject):
         r = self.getExtent()
         return mouse in r
 
-    def containsMouse(self, event):
+    def containsMouse(self, event: Event) -> bool:
         """
         Returns True if a mouse event occurs inside the calling view, otherwise
         returns False. Returns True if the view is visible and the mouse
@@ -304,7 +312,7 @@ class View(VindaugaObject):
 
         return (self.state & sfVisible) and self.mouseInView(event.mouse.where)
 
-    def locate(self, bounds):
+    def locate(self, bounds: Rect):
         """
         Changes the bounds of the view to those of the `bounds` argument.
         The view is redrawn in its new location.
@@ -334,7 +342,7 @@ class View(VindaugaObject):
                     r.bottomRight += SHADOW_SIZE
                 self.drawUnderRect(r, None)
 
-    def dragView(self, event, mode, limits, minSize, maxSize):
+    def dragView(self, event: Event, mode: int, limits: Rect, minSize: Point, maxSize: Point):
         """
         Drags the view in the ways specified by the `mode` argument, that is
         interpreted like the `growMode` data member.
@@ -361,7 +369,7 @@ class View(VindaugaObject):
 
         self.setState(sfDragging, False)
 
-    def calcBounds(self, delta):
+    def calcBounds(self, delta: Point) -> Rect:
         """
         When a view's owner changes size, the owner repeatedly calls
         `calcBounds()` and `changeBounds()` for all its subviews.
@@ -373,7 +381,7 @@ class View(VindaugaObject):
         `calcBounds()` calculates the new bounds using the flags specified
         in `growMode` data member.
 
-        :param delta: `Rect` with the change in size
+        :param delta: `Point` with the change in size
         :return: `Rect` with new bounds
         """
         bounds = self.getBounds()
@@ -402,7 +410,7 @@ class View(VindaugaObject):
         bounds.bottomRight.y = bounds.topLeft.y + clamp(bounds.bottomRight.y - bounds.topLeft.y, minLim.y, maxLim.y)
         return bounds
 
-    def changeBounds(self, bounds):
+    def changeBounds(self, bounds: Rect):
         """
         `changeBounds()` changes the view's bounds (`origin` and `size`
         data members) to the rectangle given by the `bounds` parameter.
@@ -418,7 +426,7 @@ class View(VindaugaObject):
         self.setBounds(bounds)
         self.drawView()
 
-    def growTo(self, x, y):
+    def growTo(self, x: int, y: int):
         """
         Grows or shrinks the view to the given size using a call to `locate().`
 
@@ -428,7 +436,7 @@ class View(VindaugaObject):
         r = Rect(self.origin.x, self.origin.y, self.origin.x + x, self.origin.y + y)
         self.locate(r)
 
-    def moveTo(self, x, y):
+    def moveTo(self, x: int, y: int):
         """
         Moves the origin to the point (x,y) relative to the owner's view. The view's size is unchanged.
 
@@ -438,7 +446,7 @@ class View(VindaugaObject):
         r = Rect(x, y, x + self.size.x, y + self.size.y)
         self.locate(r)
 
-    def setBounds(self, bounds):
+    def setBounds(self, bounds: Rect):
         """
         Sets the bounding rectangle of the view to the value given by the
         `bounds` parameter. The `origin` data member is set to `bounds.topLeft',
@@ -452,7 +460,7 @@ class View(VindaugaObject):
         self.origin = Point(bounds.topLeft.x, bounds.topLeft.y)
         self.size = bounds.bottomRight - bounds.topLeft
 
-    def getHelpCtx(self):
+    def getHelpCtx(self) -> int:
         """
         `getHelpCtx()` returns the view's help context. The default `getHelpCtx()`
         returns the value in the `helpCtx` data member, or returns `hcDragging` if the view is being dragged
@@ -541,7 +549,7 @@ class View(VindaugaObject):
             self.draw()
             self.drawCursor()
 
-    def exposedChildren(self, left, right, siblings, context):
+    def exposedChildren(self, left: int, right: int, siblings: List[View], context: TargetContext):
         for i, child in enumerate(siblings):
             if child is context.target:
                 return self.exposedParent(left, right, child.owner)
@@ -566,21 +574,21 @@ class View(VindaugaObject):
                     if left >= right:
                         return False
 
-    def exposedParent(self, left, right, item):
-        if not (item.state & sfVisible):
+    def exposedParent(self, left: int, right: int, child: View):
+        if not (child.state & sfVisible):
             return False
 
-        if (not item.owner) or item.owner.buffer:
+        if (not child.owner) or child.owner.buffer:
             return True
 
         with self.context as context:
-            context.y += item.origin.y
+            context.y += child.origin.y
 
-            left += item.origin.x
-            right += item.origin.x
-            context.target = item
+            left += child.origin.x
+            right += child.origin.x
+            context.target = child
 
-            group = item.owner
+            group = child.owner
             # Is my parent overlapping me?
             if not group.clip.topLeft.y <= context.y < group.clip.bottomRight.y:
                 return False
@@ -593,7 +601,7 @@ class View(VindaugaObject):
 
             return self.exposedChildren(left, right, group.children, context)
 
-    def exposed(self):
+    def exposed(self) -> bool:
         """
         Checks if the view is exposed.
 
@@ -611,7 +619,7 @@ class View(VindaugaObject):
 
         return any(self.exposedParent(0, self.size.x, self) for self.context.y in range(self.size.y))
 
-    def focus(self):
+    def focus(self) -> bool:
         """
         Tries to grab the focus.
 
@@ -646,7 +654,7 @@ class View(VindaugaObject):
         """
         self.setState(sfCursorVis, False)
 
-    def drawHide(self, lastView):
+    def drawHide(self, lastView: Optional[View]):
         """
         Calls `drawCursor()` followed by `drawUnderView()`. The latter
         redraws all subviews (with shadows if required) until the given
@@ -657,7 +665,7 @@ class View(VindaugaObject):
         self.drawCursor()
         self.drawUnderView(bool(self.state & sfShadow), lastView)
 
-    def drawShow(self, lastView):
+    def drawShow(self, lastView: Optional[View]):
         """
         Calls `drawView()`, then if `state` data member has the
         `sfShadow` bit set, `drawUnderView()` is called to draw the
@@ -669,7 +677,7 @@ class View(VindaugaObject):
         if self.state & sfShadow:
             self.drawUnderView(True, lastView)
 
-    def drawUnderRect(self, r, lastView):
+    def drawUnderRect(self, r: Rect, lastView: Optional[View]):
         """
         Calls `owner.clip.intersect(r)` to set the area that needs drawing.
         Then, all the subviews from the next view to the given `lastView` are
@@ -683,7 +691,7 @@ class View(VindaugaObject):
         self.owner.drawSubViews(self.nextView(), lastView)
         self.owner.clip = self.owner.getExtent()
 
-    def drawUnderView(self, doShadow, lastView):
+    def drawUnderView(self, doShadow: bool, lastView: Optional[View]):
         """
         Calls `drawUnderRect(r, lastView)`, where `r` is the calling view's
         current bounds. If `doShadow` is True, the view's bounds are first
@@ -697,7 +705,7 @@ class View(VindaugaObject):
             r.bottomRight += SHADOW_SIZE
         self.drawUnderRect(r, lastView)
 
-    def consumesData(self):
+    def consumesData(self) -> bool:
         """
         Any non-zero returns indicates the view can get and set data
         :return:
@@ -739,48 +747,12 @@ class View(VindaugaObject):
 
     def resetCursor(self):
         """
-        Resets the cursor
+        Resets the cursor using the Cursor class
         """
-        if self.state & (sfVisible | sfCursorVis | sfFocused) == (sfVisible | sfCursorVis | sfFocused):
-            p = self
-            cur = Point()
-            cur.x = self.cursor.x
-            cur.y = self.cursor.y
+        cursor_instance = Cursor()
+        cursor_instance.reset_cursor(self)
 
-            while True:
-                if not (0 <= cur.x < p.size.x and 0 <= cur.y < p.size.y):
-                    break
-
-                cur.x += p.origin.x
-                cur.y += p.origin.y
-                p2 = p
-                owner = p.owner
-                if not owner:
-                    if self.state & sfCursorIns:
-                        Screen.setBigCursor()
-                    else:
-                        Screen.setSmallCursor()
-                    Screen.moveCursor(cur.x, cur.y)
-                    Screen.drawCursor(1)
-                    return
-
-                if not (owner.state & sfVisible):
-                    break
-
-                for p in owner.children:
-                    if p == p2:
-                        p = p.owner
-                        # continue outer loop
-                        break
-
-                    if ((p.state & sfVisible) and
-                            p.origin.x <= cur.x < p.size.x + p.origin.x and
-                            p.origin.y <= cur.y < p.size.y + p.origin.y):
-                        Screen.drawCursor(0)
-                        return
-        Screen.drawCursor(0)
-
-    def setCursor(self, x, y):
+    def setCursor(self, x: int, y: int):
         """
         Moves the (virtual) hardware cursor to the point (x, y) using view-relative
         (local) coordinates. (0, 0) is the top-left corner.
@@ -818,7 +790,7 @@ class View(VindaugaObject):
         event.clear(self)
         event.what = evNothing
 
-    def eventAvail(self):
+    def eventAvail(self) -> bool:
         """
         Calls `getEvent()` and returns True if an event is available. Calls
         `putEvent()` to set the event as pending.
@@ -873,7 +845,7 @@ class View(VindaugaObject):
                 if not self.focus() or not (self.options & ofFirstClick):
                     self.clearEvent(event)
 
-    def putEvent(self, event):
+    def putEvent(self, event: Event):
         """
         Puts the event given by `event` into the event queue, causing it to be
         available to be returned by `getEvent()`.
@@ -894,6 +866,13 @@ class View(VindaugaObject):
         if self.owner:
             self.owner.putEvent(event)
 
+    def unputEvent(self, event: Event):
+        """
+        Puts the event given by `event` to the front of the event queue
+        """
+        if self.owner:
+            self.owner.unputEvent(event)
+
     def endModal(self, command: int):
         """
         Calls `TopView()` to seek the top most modal view. If there is none
@@ -909,7 +888,7 @@ class View(VindaugaObject):
         if self.TopView:
             self.TopView.endModal(command)
 
-    def execute(self):
+    def execute(self) -> int:
         """
         Is called from `Group.execView()` whenever a view becomes modal.
         If a view is to allow modal execution, it must override `execute()` to
@@ -921,7 +900,7 @@ class View(VindaugaObject):
         """
         return cmCancel
 
-    def getColor(self, color):
+    def getColor(self, color) -> AttributePair:
         """
         Maps the palette indices in the low and high bytes of `color` into
         physical character attributes by tracing through the palette of the
@@ -930,13 +909,19 @@ class View(VindaugaObject):
         :param color: Color to map
         :return: Color pair
         """
-        colorPair = (color >> 8) & 0xFF
-        if colorPair:
-            colorPair = self.mapColor(colorPair) << 8
-        colorPair |= self.mapColor(color & 0xFF)
-        return colorPair
+        # Extract high byte (highlight attribute) and low byte (normal attribute)
+        high_index = (color >> 8) & 0xFF
+        low_index = color & 0xFF
 
-    def getPalette(self):
+        # Map both indices through the palette system
+        high_attr = self.mapColor(high_index) if high_index != 0 else ColourAttribute()
+        low_attr = self.mapColor(low_index)
+
+        # Create AttributePair with mapped colors
+        return AttributePair(pair=(low_attr, high_attr))
+
+
+    def getPalette(self) -> Palette:
         """
         `getPalette()` returns `Palette` object representing the view's palette.
 
@@ -950,9 +935,9 @@ class View(VindaugaObject):
 
         :return: `Palette` object
         """
-        return Palette('')
+        return Palette([])
 
-    def mapColor(self, color):
+    def mapColor(self, index: int) -> ColourAttribute:
         """
         Maps the given color to an offset into the current palette. `mapColor()`
         works by calling `getPalette()` for each owning group in the chain.
@@ -966,23 +951,24 @@ class View(VindaugaObject):
         :param color: Color to map
         :return: Color
         """
-        if not color:
-            return self.errorAttr
+        palette = self.getPalette()
+        color = ColourAttribute()
 
-        current = self
-        while current:
-            palette = current.getPalette()
-            paletteLen = len(palette)
-            if paletteLen:
-                if color > paletteLen:
-                    return self.errorAttr
-                color = palette[color]
-                if color == 0:
-                    return self.errorAttr
-            current = current.owner
+        if len(palette):
+            if 0 < index <= len(palette):
+                color = palette[index]
+            else:
+                return ColourAttribute.from_bios(self.errorAttr)
+        else:
+            color = ColourAttribute.from_bios(index)
+
+        if int(color) == 0:
+            return ColourAttribute.from_bios(self.errorAttr)
+        if self.owner:
+            return self.owner.mapColor(int(color))  # .to_bios())
         return color
 
-    def getState(self, state):
+    def getState(self, state) -> bool:
         """
         Returns True if the state given in `state` is set in the data member
         `state.`
@@ -1008,7 +994,7 @@ class View(VindaugaObject):
         elif self.owner:
             self.owner.setCurrent(self, self.normalSelect)
 
-    def setState(self, state, enable):
+    def setState(self, state: int, enable: bool):
         """
         Sets or clears a state flag in the `state` data member.
         The `state` parameter specifies the state flag to modify, and the
@@ -1056,7 +1042,7 @@ class View(VindaugaObject):
             command = cmReceivedFocus if enable else cmReleasedFocus
             message(self.owner, evBroadcast, command, self)
 
-    def keyEvent(self, event):
+    def keyEvent(self, event: Event):
         """
         Returns, in the `event` variable, the next `evKeyDown` event.
         It waits, ignoring all other events, until a keyboard event becomes
@@ -1069,7 +1055,7 @@ class View(VindaugaObject):
             self.getEvent(event)
             keyIsDown = (event.what == evKeyDown)
 
-    def mouseEvent(self, event, mask):
+    def mouseEvent(self, event: Event, mask: int) -> bool:
         """
         Sets the next mouse event in the `event` argument.
         Returns True if this event is in the `mask` argument. Also returns
@@ -1089,7 +1075,7 @@ class View(VindaugaObject):
             mouseIsMasked = not (event.what & mask)
         return event.what != evMouseUp
 
-    def makeGlobal(self, source):
+    def makeGlobal(self, source: Point) -> Point:
         """
         Converts the `source` point coordinates from local (view) to global
         (screen) and returns the result.
@@ -1104,7 +1090,7 @@ class View(VindaugaObject):
             temp += cur.origin
         return temp
 
-    def makeLocal(self, source):
+    def makeLocal(self, source: Point) -> Point:
         """
         Converts the `source` point coordinates from global (screen) to local
         (view) and returns the result.
@@ -1122,7 +1108,7 @@ class View(VindaugaObject):
             temp -= cur.origin
         return temp
 
-    def nextView(self):
+    def nextView(self) -> Optional[View]:
         """
         Returns the next subview in the owner's subview list. None is returned if the
         calling view is the last one in its owner's list.
@@ -1133,7 +1119,7 @@ class View(VindaugaObject):
             return None
         return self.next
 
-    def prevView(self):
+    def prevView(self) -> Optional[View]:
         """
         Returns the previous subview in the owner's subview list. None is returned if the
         calling view is the first one in its owner's list.
@@ -1151,7 +1137,7 @@ class View(VindaugaObject):
         """
         self.putInFrontOf(self.owner.first)
 
-    def putInFrontOf(self, target):
+    def putInFrontOf(self, target: View):
         """
         Moves the calling view in front of the `target` view in the owner's
         subview list. The call
@@ -1197,7 +1183,7 @@ class View(VindaugaObject):
                 if self.options & ofSelectable:
                     self.owner.resetCurrent()
 
-    def writeBuf(self, x, y, w, h, buf):
+    def writeBuf(self, x: int, y: int, w: int, h: int, buf):
         """
         Writes the given buffer to the screen starting at the coordinates
         (x, y), and filling the region of width `w` and height `h`. Should only
@@ -1209,16 +1195,11 @@ class View(VindaugaObject):
         :param h: Height
         :param buf: Buffer
         """
+        for row in range(h):
+            self.__writeView(x, x + w, y + row, buf[row * w: w * (row + 1)])
+        # self.doRefresh()
 
-        Screen.lockRefresh += 1
-        try:
-            for row in range(h):
-                self.__writeView(x, x + w, y + row, buf[row * w: w * (row + 1)])
-        finally:
-            Screen.lockRefresh -= 1
-        self.doRefresh()
-
-    def writeChar(self, x, y, c, color, count):
+    def writeChar(self, x: int, y: int, c: int, color: int, count: int):
         """
         Beginning at the point (x, y), writes `count` copies of the character
         `c` in the color determined by the color'th entry in the current view's
@@ -1230,17 +1211,15 @@ class View(VindaugaObject):
         :param color: color
         :param count: count
         """
-        if x < 0:
-            x = 0
-
-        if x + count > LINE_WIDTH:
+        if count <= 0:
             return
 
-        b = DrawBuffer()
-        b.moveChar(0, c, self.mapColor(color), count)
-        self.__writeView(x, x + count, y, b)
+        cell = ScreenCell()
+        set_cell(cell, x, y, self.mapColor(color))
+        buf = [cell] * count
+        self.__writeView(x, x + count, y, buf)
 
-    def writeLine(self, x, y, w, h, buf):
+    def writeLine(self, x: int, y: int, w: int, h: int, buf):
         """
         Writes the line contained in the buffer `b` to the screen, beginning at
         the point (x, y) within the rectangle defined by the width `w` and the
@@ -1256,15 +1235,11 @@ class View(VindaugaObject):
         if not h:
             return
         width = x + w
-        Screen.lockRefresh += 1
-        try:
-            for row in range(y, y + h):
-                self.__writeView(x, width, row, buf)
-        finally:
-            Screen.lockRefresh -= 1
-        self.doRefresh()
+        for row in range(y, y + h):
+            self.__writeView(x, width, row, buf)
+        # self.doRefresh()
 
-    def writeStr(self, x, y, text, color):
+    def writeStr(self, x: int, y: int, text: str, color: int):
         """
         Writes the string `text` with the color attributes of the color'th entry
         in the view's palette, beginning at the point (x, y). Should only be
@@ -1278,15 +1253,17 @@ class View(VindaugaObject):
         if not text:
             return
 
-        textLen = len(text)
-        b = DrawBuffer()
-        b.moveStr(0, text, color)
-        self.__writeView(x, x + textLen, y, b)
+        textLen = wcwidth.wcswidth(text)
+        attr = self.mapColor(color)
+        buf = [ScreenCell() for _ in range(textLen)]
+        for b, s in zip(buf, text):
+            set_cell(b, s, attr)
+        self.__writeView(x, x + textLen, y, buf)
 
-    def at(self, index):
+    def at(self, index: int) -> View:
         return self.children[index]
 
-    def _grow(self, initial, size, delta):
+    def _grow(self, initial: int, size: int, delta: int) -> int:
         if self.growMode & gfGrowRel:
             initial = (initial * size + ((size - delta) >> 1)) // (size - delta)
         else:
@@ -1294,7 +1271,7 @@ class View(VindaugaObject):
         return initial
 
     @staticmethod
-    def __change(mode, delta, point, size, ctrlState):
+    def __change(mode: int, delta: Point, point: Point, size: Point, ctrlState: int):
         """
         Alter point and size depending on the drag/move mode
 
@@ -1311,7 +1288,7 @@ class View(VindaugaObject):
             size += delta
         return point, size
 
-    def __writeChildrenViewRec(self, left, right, children, shadowCounter, context):
+    def __writeChildrenViewRec(self, left: int, right: int, children: List[View], shadowCounter: int, context: TargetContext):
         for i, view in enumerate(children):
             if view is context.target:
                 if view.owner.buffer:
@@ -1320,8 +1297,6 @@ class View(VindaugaObject):
                     else:  # Paint with shadow
                         self.__paintWithShadow(context, view, left, right)
 
-                    if view.owner.buffer is Screen.screenBuffer:
-                        Screen.drawMouse(True)
                 if not view.owner.lockFlag:
                     self.__writeViewRec2(left, right, view.owner, shadowCounter)
                 return
@@ -1376,9 +1351,9 @@ class View(VindaugaObject):
                     left = view.origin.x + view.size.x + SHADOW_SIZE.x
                     shadowCounter -= 1
 
-    def __writeViewRec2(self, left, right, view, shadowCounter):
+    def __writeViewRec2(self, left: int, right: int, view: View, shadowCounter: int):
         if not (view.state & sfVisible) or not view.owner:
-            return
+            return False
 
         with self.context as context:
             context.y += view.origin.y
@@ -1390,17 +1365,18 @@ class View(VindaugaObject):
             group = view.owner
 
             if not (group.clip.topLeft.y <= context.y < group.clip.bottomRight.y):
-                return
+                return False
 
             left = max(left, group.clip.topLeft.x)
             right = min(right, group.clip.bottomRight.x)
 
             if left >= right:
-                return
+                return False
 
             self.__writeChildrenViewRec(left, right, group.children, shadowCounter, context)
+            return True
 
-    def __writeView(self, left, right, y, buf):
+    def __writeView(self, left: int, right: int, y: int, buf):
 
         if not (0 <= y < self.size.y):
             return
@@ -1416,10 +1392,10 @@ class View(VindaugaObject):
             context.y = y
 
             self.savedBuffer = buf
-            self.__writeViewRec2(left, right, self, 0)
-            self.doRefresh()
+            if self.__writeViewRec2(left, right, self, 0):
+                self.doRefresh()
 
-    def __moveGrow(self, point, size, limits, minSize, maxSize, mode):
+    def __moveGrow(self, point: Point, size: Point, limits: Rect, minSize: Point, maxSize: Point, mode: int):
         size.x = min(max(size.x, minSize.x), maxSize.x)
         size.y = min(max(size.y, minSize.y), maxSize.y)
 
@@ -1441,55 +1417,71 @@ class View(VindaugaObject):
         r = Rect(point.x, point.y, point.x + size.x, point.y + size.y)
         self.locate(r)
 
-    def __paintWithoutShadow(self, context, view, left, right):
-        width = (right - left)
+    def __paintWithoutShadow(self, context, view: View, left: int, right: int):
+        width = right - left
         pOwner = view.owner
         offset = context.offset
         soff = left - offset
-        if pOwner.buffer is Screen.screenBuffer:
+        if pOwner.buffer is Screen.screen.screenBuffer:
             # Write something to the screen.
-            Screen.writeRow(left, context.y, self.savedBuffer[soff: soff + width], width)
+            systemInterface.screenWrite(left, context.y, self.savedBuffer[soff: soff + width], width)
 
         poff = pOwner.size.x * context.y + left
-        pOwner.buffer[poff: poff + width] = BufferArray(self.savedBuffer[soff: soff + width])
+        # Use individual assignments instead of slice assignment to avoid buffer size corruption
+        for i in range(width):
+            if poff + i < len(pOwner.buffer) and soff + i < len(self.savedBuffer):
+                pOwner.buffer[poff + i] = copy.deepcopy(self.savedBuffer[soff + i])
 
-    def __paintWithShadow(self, context, view, left, right):
+    def __paintWithShadow(self, context, view: View, left: int, right: int):
         width = right - left
         dst1 = view.owner.size.x * context.y + left
         start = left - context.offset
-        src = self.savedBuffer[start: start + width]
+        src = itertools.islice(self.savedBuffer, start, start + width)
         for offset, d in enumerate(src):
-            d = d & DrawBuffer.CHAR_MASK | (SHADOW_ATTR << DrawBuffer.CHAR_WIDTH)
-            if view.owner.buffer is Screen.screenBuffer:
-                Screen.writeRow(offset + left, context.y, [d], 1)
+            # Create shadow cell - keep character, set shadow attribute
+            shadow_cell: ScreenCell = copy.deepcopy(d)
+            # special handling for the wallpaper character...
+            if shadow_cell.char == '▄':
+                shadow_cell.char = '▒'
+
+            set_attr(shadow_cell, SHADOW_ATTR)
+            d = shadow_cell
+            if view.owner.buffer is Screen.screen.screenBuffer:
+                systemInterface.screenWrite(offset + left, context.y, [d], 1)
             view.owner.buffer[dst1 + offset] = d
 
-    def __handleMouseDownDrag(self, event, mode, limits, minSize, maxSize):
+    def __handleMouseDownDrag(self, event: Event, mode: int, limits: Rect, minSize: Point, maxSize: Point):
         """
         Handle mouse down event
         """
         if mode & dmDragMove:
-            point = self.origin - event.mouse.where
+            initialMousePos = event.mouse.where
+            initialOrigin = self.origin
+            point = initialOrigin - initialMousePos
             working = True
             while working:
-                event.mouse.where += point
-                self.__moveGrow(event.mouse.where, self.size, limits, minSize, maxSize, mode)
+                # Calculate new origin based on current mouse position and initial offset
+                newOrigin = event.mouse.where + point
+                self.__moveGrow(newOrigin, self.size, limits, minSize, maxSize, mode)
                 working = self.mouseEvent(event, evMouseMove)
             # Pop the window to the mouse-up coords
-            event.mouse.where += point
-            self.__moveGrow(event.mouse.where, self.size, limits, minSize, maxSize, mode)
+            newOrigin = event.mouse.where + point
+            self.__moveGrow(newOrigin, self.size, limits, minSize, maxSize, mode)
         else:
-            point = self.size - event.mouse.where
+            initialMousePos = event.mouse.where
+            initialSize = self.size
+            point = initialSize - initialMousePos
             working = True
             while working:
-                event.mouse.where += point
-                self.__moveGrow(self.origin, event.mouse.where, limits, minSize, maxSize, mode)
+                # Calculate new size based on current mouse position and initial offset
+                newSize = event.mouse.where + point
+                self.__moveGrow(self.origin, newSize, limits, minSize, maxSize, mode)
                 working = self.mouseEvent(event, evMouseMove)
             # Grow to the mouse-up
-            event.mouse.where += point
-            self.__moveGrow(self.origin, event.mouse.where, limits, minSize, maxSize, mode)
+            newSize = event.mouse.where + point
+            self.__moveGrow(self.origin, newSize, limits, minSize, maxSize, mode)
 
-    def __handleKeyDownDrag(self, event, mode, limits, minSize, maxSize):
+    def __handleKeyDownDrag(self, event: Event, mode: int, limits: Rect, minSize: Point, maxSize: Point):
         """
         Handle key down event
         """
