@@ -11,11 +11,11 @@ class VindaugaObject:
 
     _registry = {}
 
-    @classmethod
     def __init_subclass__(cls, **kwargs):
         super().__init_subclass__()
         try:
-            VindaugaObject._registry[cls.name] = cls
+            if name := getattr(cls, 'name', None):
+                VindaugaObject._registry[name] = cls
         except AttributeError:
             logger.info('A class has no name: %s', cls)
 
