@@ -7,15 +7,18 @@ from vindauga.constants.event_codes import evBroadcast, evMouseDown, evKeyDown,
 from vindauga.constants.option_flags import ofSelectable, ofFirstClick, ofPreProcess, ofPostProcess
 from vindauga.constants.state_flags import sfActive, sfSelected, sfFocused, sfDisabled
 from vindauga.events.event import Event
-from vindauga.misc.character_codes import SPECIAL_CHARS, getAltCode
-from vindauga.misc.cp437 import cp437ToUnicode
-from vindauga.misc.message import message
-from vindauga.misc.util import hotKey
-from vindauga.misc.util import nameLength
+from vindauga.utilities.input.character_codes import SPECIAL_CHARS, getAltCode
+from vindauga.utilities.message import message
+from vindauga.utilities.text.string_utils import hotKey
+from vindauga.utilities.text.string_utils import nameLength
+from vindauga.utilities.colours.attribute_pair import AttributePair
 from vindauga.types.draw_buffer import DrawBuffer
+from vindauga.types.group import Phases
 from vindauga.types.palette import Palette
+from vindauga.types.rect import Rect
 from vindauga.types.view import View
 
+
 logger = logging.getLogger(__name__)
 
 cmGrabDefault = 61
@@ -28,15 +31,13 @@ class Button(View):
     cpButton = "\x0A\x0B\x0C\x0D\x0E\x0E\x0E\x0F"
     markers = '[]'
 
-    def __init__(self, bounds, title, commands, flags):
+    def __init__(self, bounds: Rect, title: str, commands: int, flags: int):
         super().__init__(bounds)
         self.title = title
         self._command = commands
         self._flags = flags
         self._amDefault = ((flags & bfDefault) != 0)
-
         self.options |= (ofSelectable | ofFirstClick | ofPreProcess | ofPostProcess)
-
         self.eventMask |= evBroadcast
 
         if not self.commandEnabled(commands):
@@ -45,7 +46,7 @@ class Button(View):
     def draw(self):
         self.drawState(False)
 
-    def drawState(self, down):
+    def drawState(self, down: bool):
         b = DrawBuffer()
         ch = ' '
 
@@ -76,9 +77,9 @@ class Button(View):
                     ch = ' '
                 else:
                     if y == 0:
-                        b.putChar(s, self.shadows[0])
+                        b.putChar(s, self.shadows[0], cShadow)
                     else:
-                        b.putChar(s, self.shadows[1])
+                        b.putChar(s, self.shadows[1], cShadow)
 
                     ch = self.shadows[2]
                 i = 1
@@ -87,8 +88,8 @@ class Button(View):
                 self.__drawTitle(b, s, i, cButton, down)
 
             if self.showMarkers and down:
-                b.putChar(1, self.markers[0])
-                b.putChar(s - 1, self.markers[1])
+                b.putChar(1, self.markers[0], cButton)
+                b.putChar(s - 1, self.markers[1], cButton)
 
             self.writeLine(0, y, self.size.x, 1, b)
 
@@ -97,11 +98,11 @@ class Button(View):
 
         self.writeLine(0, self.size.y - 1, self.size.x, 1, b)
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         return Palette(self.cpButton)
 
-    def handleEvent(self, event):
-        clickRect = self.getExtent()
+    def handleEvent(self, event: Event):
+        clickRect: Rect = self.getExtent()
         clickRect.topLeft.x += 1
         clickRect.bottomRight.x -= 1
         clickRect.bottomRight.y -= 1
@@ -134,7 +135,7 @@ class Button(View):
 
         elif event.what == evKeyDown:
             if ((event.keyDown.keyCode == getAltCode(c)) or
-                    (self.owner.phase == self.phPostProcess and c and event.keyDown.charScan.charCode.upper() == c) or
+                    (self.owner.phase == Phases.Postprocess and c and event.keyDown.charScan.charCode.upper() == c) or
                     ((self.state & sfFocused) and event.keyDown.charScan.charCode == ' ')):
                 self.press()
                 self.clearEvent(event)
@@ -155,14 +156,14 @@ class Button(View):
                 self.setState(sfDisabled, not self.commandEnabled(command))
                 self.drawView()
 
-    def makeDefault(self, enable):
+    def makeDefault(self, enable: bool):
         if not (self._flags & bfDefault):
             message(self.owner, evBroadcast,
                     [cmReleaseDefault, cmGrabDefault][enable], self)
             self._amDefault = enable
             self.drawView()
 
-    def setState(self, state, enable):
+    def setState(self, state: int, enable: bool):
         super().setState(state, enable)
 
         if state & (sfSelected | sfActive):
@@ -186,7 +187,7 @@ class Button(View):
             e.message.infoPtr = self
             self.putEvent(e)
 
-    def __drawTitle(self, b, s, i, cButton, down):
+    def __drawTitle(self, b: DrawBuffer, s: int, i: int, cButton: AttributePair, down: bool):
         if self._flags & bfLeftJust:
             titlePosition = 0
         else:
@@ -205,5 +206,5 @@ class Button(View):
             else:
                 scOff = 4
 
-            b.putChar(0, SPECIAL_CHARS[scOff])
-            b.putChar(s, SPECIAL_CHARS[scOff + 1])
+            b.putChar(0, SPECIAL_CHARS[scOff], cButton)
+            b.putChar(s, SPECIAL_CHARS[scOff + 1], cButton)
