@@ -1,6 +1,9 @@
 # -*- coding: utf-8 -*-
+import wcwidth
+
 from vindauga.types.draw_buffer import DrawBuffer
 from vindauga.types.palette import Palette
+from vindauga.types.rect import Rect
 from vindauga.types.view import View
 
 
@@ -8,7 +11,7 @@ class DynamicText(View):
     name = 'DynamicText'
     cpDynamicText = '\x06'
 
-    def __init__(self, bounds, text, rightJustify=True):
+    def __init__(self, bounds: Rect, text: str, rightJustify: bool = True):
         super().__init__(bounds)
         self._text = text
         self._rightJustify = rightJustify
@@ -16,24 +19,24 @@ class DynamicText(View):
     def draw(self):
         b = DrawBuffer()
         color = self.getColor(0x01)
-        offset = self.size.x - len(self._text) if self._rightJustify else 0
+        offset = self.size.x - wcwidth.wcswidth(self._text) if self._rightJustify else 0
 
         b.moveChar(0, ' ', color, self.size.x)
         b.moveStr(offset, self._text, color)
         self.writeBuf(0, 0, self.size.x, 1, b)
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         return Palette(self.cpDynamicText)
 
-    def setText(self, text):
+    def setText(self, text: str):
         self.setData(text)
         self.drawView()
 
-    def setData(self, text):
+    def setData(self, text: str):
         self._text = text
 
-    def getData(self):
+    def getData(self) -> str:
         return self._text
 
-    def consumesData(self):
+    def consumesData(self) -> bool:
         return True
