@@ -1,7 +1,11 @@
 # -*- coding: utf-8 -*-
+from __future__ import annotations
+from enum import Enum, auto
 import logging
 import string
-from typing import List, Optional
+from typing import List, Optional, Union, Tuple
+
+import wcwidth
 
 from vindauga.constants.command_codes import *
 from vindauga.constants.edit_command_codes import *
@@ -11,12 +15,13 @@ from vindauga.constants.keys import *
 from vindauga.constants.option_flags import ofSelectable
 from vindauga.constants.state_flags import sfVisible, sfCursorIns, sfActive, sfExposed
 from vindauga.events.event import Event
-from vindauga.types.records.find_dialog_record import FindDialogRecord
-from vindauga.types.records.replace_dialog_record import ReplaceDialogRecord
+from vindauga.utilities.screen.screen_cell import set_cell
+from vindauga.utilities.colours.colour_attribute import ColourAttribute
+from vindauga.utilities.text.text import Text
 from vindauga.types.command_set import CommandSet
 from vindauga.types.palette import Palette
 from vindauga.types.point import Point
-from vindauga.types.draw_buffer import BufferArray, DrawBuffer
+from vindauga.types.draw_buffer import DrawBuffer
 from vindauga.types.rect import Rect
 from vindauga.types.view import View
 from vindauga.widgets.indicator import Indicator
@@ -24,6 +29,15 @@ from vindauga.widgets.scroll_bar import ScrollBar
 
 logger = logging.getLogger(__name__)
 
+
+class Encoding(Enum):
+    encDefault = auto()
+    encSingleByte = auto()
+
+
+maxFindStrLen = 80
+maxReplaceStrLen = 80
+
 firstKeys = {
     kbCtrlA: cmWordLeft,
     kbCtrlC: cmPageDown,
@@ -87,11 +101,11 @@ blockKeys = {
 keyMaps = (firstKeys, quickKeys, blockKeys)
 
 
-def isWordChar(ch):
+def isWordChar(ch: str) -> bool:
     return ch in string.ascii_letters or ch in string.digits or ch == ' '
 
 
-def scanKeyMap(keyMap, keyCode):
+def scanKeyMap(keyMap: dict, keyCode: Union[int, str]) -> int:
     if isinstance(keyCode, str):
         keyCode = ord(keyCode)
     codeHi, codeLow = divmod(keyCode, 256)
@@ -116,8 +130,7 @@ class Editor(View):
 
         self.delta = Point(0, 0)
         self.limit = Point(0, 0)
-        self._currentPosition = Point(0, 0)
-        self.hScrollBar = hScrollBar
+        self.curPos = Point(0, 0)
         self.vScrollBar = vScrollBar
         self.indicator = indicator
 
@@ -136,7 +149,7 @@ class Editor(View):
         self.insCount = 0
         self.updateFlags = 0
         self.drawPtr = 0
-        self.drawLine = 0
+        self.drawLineNum = 0
         self.lockCount = 0
         self.keyState = 0
         self.editorFlags = efBackupFiles | efPromptOnReplace
@@ -154,29 +167,59 @@ class Editor(View):
         self.showCursor()
         self.initBuffer(bufSize)
         self.isValid = True
+        self.encoding = Encoding.encDefault
         self.setBufLen(0)
 
     @staticmethod
-    def editorDialog(_flag, *_args):
+    def editorDialog(_flag, *_args) -> int:
         logger.info('Unreplaced editorDialog')
         return cmCancel
 
-    def bufChar(self, pos):
-        o = 0
-        if pos > self.curPtr:
-            o = self.gapLen
-        return chr(self.buffer[pos + o])
+    def prevBufChars(self, pos: int) -> str:
+        if self.encoding == Encoding.encSingleByte:
+            return self.bufChar(pos - 1)
 
-    def bufPtr(self, pos):
-        if pos >= self.curPtr:
-            return pos + self.gapLen
-        return pos
+        # For string buffer, get characters before position
+        length = min(pos, 4)  # Max UTF-8 char length
+        chars = []
+
+        for i in range(length):
+            if pos - length + i >= 0:
+                chars.append(self.bufChar(pos - length + i))
+
+        return ''.join(chars)
+
+    def bufChars(self, pos: int) -> str:
+        """
+        Get string from buffer starting at pos.
+        """
+        if self.encoding == Encoding.encSingleByte:
+            return self.bufChar(pos)
+
+        # Get remaining buffer length from position
+        remaining = min(self.bufLen - pos, 32)  # Reasonable chunk for Text methods
+        chars = []
+        for i in range(remaining):
+            if pos + i < self.bufLen:
+                chars.append(self.bufChar(pos + i))
+        return ''.join(chars)
+
+    def bufChar(self, pos: int) -> str:
+        """
+        Get character at logical position.
+        """
+        if pos >= self.bufLen:
+            return '\0'
+        return self.buffer[self.bufPtr(pos)]
+
+    def bufPtr(self, pos: int) -> int:
+        return pos if pos < self.curPtr else pos + self.gapLen
 
     def shutdown(self):
         self.doneBuffer()
         super().shutdown()
 
-    def changeBounds(self, bounds):
+    def changeBounds(self, bounds: Rect):
         self.setBounds(bounds)
         self.delta.x = max(0, min(self.delta.x, self.limit.x - self.size.x))
         self.delta.y = max(0, min(self.delta.y, self.limit.y - self.size.y))
@@ -192,7 +235,7 @@ class Editor(View):
             p += 1
         return pos
 
-    def charPtr(self, p, target):
+    def charPtr(self, p: int, target: int):
         pos = 0
 
         while pos < target and p < self.bufLen and self.bufChar(p) not in ('\n', '\r'):
@@ -222,7 +265,7 @@ class Editor(View):
         if Editor.clipboard and Editor.clipboard is not self:
             self.insertFrom(Editor.clipboard)
 
-    def convertEvent(self, event):
+    def convertEvent(self, event: Event):
         if event.what == evKeyDown:
             key = event.keyDown.keyCode
             if self.keyState:
@@ -242,9 +285,9 @@ class Editor(View):
                     event.message.command = key
 
     def isCursorVisible(self) -> bool:
-        return self.delta.y <= self._currentPosition.y < (self.delta.y + self.size.y)
+        return self.delta.y <= self.curPos.y < (self.delta.y + self.size.y)
 
-    def deleteRange(self, startPtr, endPtr, delSelect):
+    def deleteRange(self, startPtr: int, endPtr: int, delSelect: bool):
         if self.hasSelection() and delSelect:
             self.deleteSelect()
         else:
@@ -254,7 +297,7 @@ class Editor(View):
             self.deleteSelect()
 
     def deleteSelect(self):
-        self.insertText([], 0, False)
+        self.insertText('', 0, False)
 
     def doneBuffer(self):
         del self.buffer
@@ -270,12 +313,12 @@ class Editor(View):
                 i = cmYes
 
                 if self.editorFlags & efPromptOnReplace:
-                    c = self.makeGlobal(self._currentPosition)
+                    c = self.makeGlobal(self.curPos)
                     i = Editor.editorDialog(edReplacePrompt, c)
                 if i == cmYes:
                     self.lock()
                     try:
-                        self.insertText(self.replaceStr, len(self.replaceStr), False)
+                        self.insertText(self.replaceStr, wcwidth.wcswidth(self.replaceStr), False)
                         self.trackCursor(False)
                     finally:
                         self.unlock()
@@ -283,11 +326,11 @@ class Editor(View):
 
     def doUpdate(self):
         if self.updateFlags:
-            self.setCursor(self._currentPosition.x - self.delta.x, self._currentPosition.y - self.delta.y)
+            self.setCursor(self.curPos.x - self.delta.x, self.curPos.y - self.delta.y)
             if self.updateFlags & ufView:
                 self.drawView()
             elif self.updateFlags & ufLine:
-                self.drawLines(self._currentPosition.y - self.delta.y, 1, self.lineStart(self.curPtr))
+                self.drawLines(self.curPos.y - self.delta.y, 1, self.lineStart(self.curPtr))
 
             if self.hScrollBar:
                 self.hScrollBar.setParams(self.delta.x, 0,
@@ -300,34 +343,32 @@ class Editor(View):
                                           self.size.y - 1, 1)
 
             if self.indicator:
-                self.indicator.setValue(self._currentPosition, self.modified)
+                self.indicator.setValue(self.curPos, self.modified)
 
             if self.state & sfActive:
                 self.updateCommands()
             self.updateFlags = 0
 
     def draw(self):
-        if self.drawLine != self.delta.y:
-            self.drawPtr = self.lineMove(self.drawPtr, self.delta.y - self.drawLine)
-            self.drawLine = self.delta.y
+        if self.drawLineNum != self.delta.y:
+            self.drawPtr = self.lineMove(self.drawPtr, self.delta.y - self.drawLineNum)
+            self.drawLineNum = self.delta.y
 
         self.drawLines(0, self.size.y, self.drawPtr)
 
-    def drawLines(self, y, count, linePtr):
+    def drawLines(self, y: int, count: int, linePtr: int):
         color = self.getColor(0x0201)
 
-        for line in range(y, y + count):
-            b = DrawBuffer(True)
-            self.formatLine(b, linePtr, self.delta.x + self.size.x, color)
-            self.writeBuf(0, line, self.size.x, 1, b[self.delta.x:])
-            linePtr = self.nextLine(linePtr)
+        for i in range(count):
+            if y + i < self.size.y:
+                b = DrawBuffer()
+                self.formatLine(b, linePtr, self.size.x, color)
+                self.writeBuf(0, y + i, self.size.x, 1, b.data)
+                linePtr = self.nextLine(linePtr)
 
     def find(self):
-        findRec = FindDialogRecord(self.findStr, self.editorFlags)
-        logger.info('find() - %s, %s', self.findStr, self.editorFlags)
         result, data = Editor.editorDialog(edFind, (self.editorFlags, self.findStr))
         if result != cmCancel:
-            logger.info('find Results: %s', data)
             self.findStr = data[1].value
             self.editorFlags = data[0].value & ~efDoReplace
             self.doSearchReplace()
@@ -339,7 +380,7 @@ class Editor(View):
         mouse.y = max(0, min(mouse.y, self.size.y - 1))
 
         return self.charPtr(self.lineMove(self.drawPtr,
-                                          mouse.y + self.delta.y - self.drawLine),
+                                          mouse.y + self.delta.y - self.drawLineNum),
                             mouse.x + self.delta.x)
 
     def getPalette(self) -> Palette:
@@ -382,10 +423,10 @@ class Editor(View):
             elif c == cmSearchAgain:
                 self.doSearchReplace()
             elif c in {
-                    cmCut, cmCopy, cmPaste, cmUndo, cmClear, cmCharLeft, cmCharRight, cmWordLeft, cmWordRight,
-                    cmLineStart, cmLineEnd, cmLineUp, cmLineDown, cmPageUp, cmPageDown, cmTextStart, cmTextEnd,
-                    cmNewLine, cmBackSpace, cmDelChar, cmDelWord, cmDelStart, cmDelEnd, cmDelLine, cmInsMode,
-                    cmStartSelect, cmHideSelect, cmIndentMode}:
+                cmCut, cmCopy, cmPaste, cmUndo, cmClear, cmCharLeft, cmCharRight, cmWordLeft, cmWordRight,
+                cmLineStart, cmLineEnd, cmLineUp, cmLineDown, cmPageUp, cmPageDown, cmTextStart, cmTextEnd,
+                cmNewLine, cmBackSpace, cmDelChar, cmDelWord, cmDelStart, cmDelEnd, cmDelLine, cmInsMode,
+                cmStartSelect, cmHideSelect, cmIndentMode, cmEncoding, cmSelectAll}:
                 self._handleEditorCommand(c, centerCursor, selectMode)
             else:
                 return
@@ -410,26 +451,30 @@ class Editor(View):
         self.setSelect(self.curPtr, self.curPtr, False)
 
     def initBuffer(self, bufSize: int):
-        self.buffer = BufferArray([0] * bufSize)
+        try:
+            # Use string buffer - simple and works
+            self.buffer = '\x00' * bufSize
+        except MemoryError:
+            self.buffer = None
 
     def countLines(self, start: int, count: int) -> int:
-        return self.buffer[start:start + count].count(ord('\n'))
+        return self.buffer[start:start + count].count('\n')
 
     @staticmethod
-    def scan(buffer: BufferArray, size: int, pattern: str) -> int:
-        result = ''.join(chr(c) for c in buffer[:size]).find(pattern)
+    def scan(buffer: str, size: int, pattern: str) -> int:
+        result = buffer.find(pattern, 0, size)
         if result < 0:
             return sfSearchFailed
         return result
 
     @staticmethod
-    def iScan(buffer: BufferArray, size: int, pattern: str) -> int:
-        result = ''.join(chr(c) for c in buffer[:size]).lower().find(pattern.lower())
+    def iScan(buffer: str, size: int, pattern: str) -> int:
+        result = buffer.lower().find(pattern.lower(), 0, size)
         if result < 0:
             return sfSearchFailed
         return result
 
-    def insertBuffer(self, p: List[int], offset: int, length: int, allowUndo: bool, selectText: bool) -> bool:
+    def insertBuffer(self, p: str, offset: int, length: int, allowUndo: bool, selectText: bool) -> bool:
         self.selecting = False
         selLen = self.selEnd - self.selStart
 
@@ -458,23 +503,37 @@ class Editor(View):
                 self.insCount -= selLen - delLen
 
             self.curPtr = self.selStart
-            self._currentPosition.y -= selLines
+            self.curPos.y -= selLines
 
-        if self.delta.y > self._currentPosition.y:
+        if self.delta.y > self.curPos.y:
             self.delta.y -= selLines
-            if self.delta.y < self._currentPosition.y:
-                self.delta.y = self._currentPosition.y
+            if self.delta.y < self.curPos.y:
+                self.delta.y = self.curPos.y
 
         if length > 0:
-            self.buffer[self.curPtr:self.curPtr + length] = BufferArray(p[offset:offset + length])
+            # Simple insertion: just overwrite part of gap with new text
+            text_to_insert = p[offset:offset + length]
+
+            # Insert text by reconstructing string (simple approach)
+            before = self.buffer[:self.curPtr]
+            gap_end = self.curPtr + self.gapLen
+            after = self.buffer[gap_end:]
+
+            new_gap_len = self.gapLen - length
+            remaining_gap = '\x00' * new_gap_len if new_gap_len > 0 else ''
+
+            self.buffer = before + text_to_insert + remaining_gap + after
+
+            self.gapLen -= length
+            self.bufLen += length
 
         lines = self.countLines(self.curPtr, length)
         self.curPtr += length
-        self._currentPosition.y += lines
+        self.curPos.y += lines
 
-        self.drawLine = self._currentPosition.y
+        self.drawLineNum = self.curPos.y
         self.drawPtr = self.lineStart(self.curPtr)
-        self._currentPosition.x = self.charPos(self.drawPtr, self.curPtr)
+        self.curPos.x = self.charPos(self.drawPtr, self.curPtr)
 
         if not selectText:
             self.selStart = self.curPtr
@@ -502,17 +561,17 @@ class Editor(View):
             self.update(ufView)
         return True
 
-    def insertFrom(self, editor):
+    def insertFrom(self, editor: Editor):
         pt = editor.bufPtr(editor.selStart)
 
         return self.insertBuffer(editor.buffer,
                                  pt, editor.selEnd - editor.selStart,
                                  self.canUndo, self.isClipboard())
 
-    def insertText(self, text, length, selectText):
-        return self.insertBuffer([ord(c) for c in text], 0, length, self.canUndo, selectText)
+    def insertText(self, text: str, length: int, selectText: bool) -> bool:
+        return self.insertBuffer(text, 0, length, self.canUndo, selectText)
 
-    def isClipboard(self):
+    def isClipboard(self) -> bool:
         return Editor.clipboard is self
 
     def lineMove(self, p: int, count: int) -> int:
@@ -546,10 +605,10 @@ class Editor(View):
                 i += 1
             self.insertText(self.buffer[p:], i - p, False)
 
-    def nextLine(self, pos):
+    def nextLine(self, pos: int) -> int:
         return self.nextChar(self.lineEnd(pos))
 
-    def nextWord(self, pos):
+    def nextWord(self, pos: int) -> int:
         while pos < self.bufLen and isWordChar(self.bufChar(pos)):
             pos = self.nextChar(pos)
         while pos < self.bufLen and not isWordChar(self.bufChar(pos)):
@@ -557,10 +616,10 @@ class Editor(View):
 
         return pos
 
-    def prevLine(self, pos):
+    def prevLine(self, pos: int) -> int:
         return self.lineStart(self.prevChar(pos))
 
-    def prevWord(self, pos):
+    def prevWord(self, pos: int) -> int:
         while pos > 0 and not isWordChar(self.bufChar(self.prevChar(pos))):
             pos = self.prevChar(pos)
         while pos > 0 and isWordChar(self.bufChar(self.prevChar(pos))):
@@ -577,7 +636,7 @@ class Editor(View):
             self.editorFlags = data[0].value | efDoReplace
             self.doSearchReplace()
 
-    def scrollTo(self, x, y):
+    def scrollTo(self, x: int, y: int):
         x = max(0, min(x, self.limit.x - self.size.x))
         y = max(0, min(y, self.limit.y - self.size.y))
         if x != self.delta.x or y != self.delta.y:
@@ -585,7 +644,7 @@ class Editor(View):
             self.delta.y = y
             self.update(ufView)
 
-    def search(self, findStr, opts):
+    def search(self, findStr: str, opts: int):
         pos = self.curPtr
         done = False
 
@@ -618,7 +677,8 @@ class Editor(View):
 
         return False
 
-    def setBufLen(self, length):
+    def setBufLen(self, length: int):
+
         self.bufLen = length
         self.gapLen = self.bufSize - length
         self.selStart = 0
@@ -626,18 +686,19 @@ class Editor(View):
         self.curPtr = 0
         self.delta.x = 0
         self.delta.y = 0
-        self._currentPosition.x = 0
-        self._currentPosition.y = 0
+        self.curPos.x = 0
+        self.curPos.y = 0
         self.limit.x = maxLineLength
         self.limit.y = self.countLines(self.gapLen, self.bufLen) + 1
-        self.drawLine = 0
+        self.drawLineNum = 0
         self.drawPtr = 0
         self.delCount = 0
         self.insCount = 0
         self.modified = False
+        self.detectEol()  # Detect line endings when buffer is set
         self.update(ufView)
 
-    def setCmdState(self, command, enable):
+    def setCmdState(self, command: int, enable: bool):
         s = CommandSet()
 
         s += command
@@ -646,7 +707,7 @@ class Editor(View):
         else:
             self.disableCommands(s)
 
-    def setCurPtr(self, pos, selectMode):
+    def setCurPtr(self, pos: int, selectMode: int):
         if not selectMode & smExtend:
             anchor = pos
         elif self.curPtr == self.selStart:
@@ -665,9 +726,51 @@ class Editor(View):
                 anchor = self.prevLine(self.nextLine(anchor))
             self.setSelect(anchor, pos, False)
 
+        # Move gap to new cursor position (critical for gap buffer!)
+        if pos != self.curPtr:
+            if pos > self.curPtr:
+                # Moving forward - move text from after gap to before gap
+                l = pos - self.curPtr
+                self.memmove(self.curPtr, self.curPtr + self.gapLen, l)
+                self.curPos.y += self.countLines(self.curPtr, l)
+                self.curPtr = pos
+            else:
+                # Moving backward - move text from before gap to after gap
+                l = self.curPtr - pos
+                self.curPtr = pos
+                self.curPos.y -= self.countLines(self.curPtr, l)
+                self.memmove(self.curPtr + self.gapLen, self.curPtr, l)
+
+            self.delCount = 0
+            self.insCount = 0
+
+        self.drawLineNum = self.curPos.y
+        self.drawPtr = self.lineStart(pos)
+        self.curPos.x = self.charPos(self.drawPtr, pos)
+
     def memmove(self, destination: int, source: int, length: int):
+        """
+        Move data within buffer - reconstruct string since strings are immutable.
+        """
+        if length <= 0:
+            return
+
+        # Get the data to move
         blob = self.buffer[source: source + length]
-        self.buffer[destination: destination + length] = blob
+
+        # Reconstruct the buffer with the moved data
+        if destination < source:
+            # Moving backwards
+            self.buffer = (self.buffer[:destination] +
+                          blob +
+                          self.buffer[destination:source] +
+                          self.buffer[source + length:])
+        else:
+            # Moving forwards
+            self.buffer = (self.buffer[:source] +
+                          self.buffer[source + length:destination + length] +
+                          blob +
+                          self.buffer[destination + length:])
 
     def setSelect(self, newStart: int, newEnd: int, curStart: bool):
         if curStart:
@@ -685,17 +788,17 @@ class Editor(View):
             if pos > self.curPtr:
                 length = pos - self.curPtr
                 self.memmove(self.curPtr, self.curPtr + self.gapLen, length)
-                self._currentPosition.y += self.countLines(self.curPtr, length)
+                self.curPos.y += self.countLines(self.curPtr, length)
                 self.curPtr = pos
             else:
                 length = self.curPtr - pos
                 self.curPtr = pos
-                self._currentPosition.y -= self.countLines(self.curPtr, length)
+                self.curPos.y -= self.countLines(self.curPtr, length)
                 self.memmove(self.curPtr + self.gapLen, self.curPtr, length)
 
-            self.drawLine = self._currentPosition.y
+            self.drawLineNum = self.curPos.y
             self.drawPtr = self.lineStart(pos)
-            self._currentPosition.x = self.charPos(self.drawPtr, pos)
+            self.curPos.x = self.charPos(self.drawPtr, pos)
             self.delCount = 0
             self.insCount = 0
             self.setBufSize(self.bufLen)
@@ -717,6 +820,8 @@ class Editor(View):
         elif state == sfExposed:
             if enable:
                 self.unlock()
+                # Force full redraw when exposed (e.g., after window resize)
+                self.update(ufView)
 
     def startSelect(self):
         self.hideSelect()
@@ -728,11 +833,11 @@ class Editor(View):
 
     def trackCursor(self, center: bool):
         if center:
-            self.scrollTo(self._currentPosition.x - self.size.x + 1,
-                          self._currentPosition.y - self.size.y // 2)
+            self.scrollTo(self.curPos.x - self.size.x + 1,
+                          self.curPos.y - self.size.y // 2)
         else:
-            self.scrollTo(max(self._currentPosition.x - self.size.x + 1, min(self.delta.x, self._currentPosition.x)),
-                          max(self._currentPosition.y - self.size.y + 1, min(self.delta.y, self._currentPosition.y)))
+            self.scrollTo(max(self.curPos.x - self.size.x + 1, min(self.delta.x, self.curPos.x)),
+                          max(self.curPos.y - self.size.y + 1, min(self.delta.y, self.curPos.y)))
 
     def undo(self):
         if self.delCount or self.insCount:
@@ -764,7 +869,7 @@ class Editor(View):
             self.setCmdState(cmPaste,
                              Editor.clipboard and (Editor.clipboard.hasSelection()))
 
-        self.setCmdState(cmClear, self.hasSelection)
+        self.setCmdState(cmClear, self.hasSelection())
         self.setCmdState(cmFind, True)
         self.setCmdState(cmReplace, True)
         self.setCmdState(cmSearchAgain, True)
@@ -773,67 +878,117 @@ class Editor(View):
         return self.isValid
 
     def lineStart(self, pos: int) -> int:
-        while pos > self.curPtr:
-            pos -= 1
-            if self.buffer[pos + self.gapLen] == ord('\n'):
-                return pos + 1
-        if self.curPtr == 0:
-            return 0
-
         while pos > 0:
-            pos -= 1
-            if self.buffer[pos] == ord('\n'):
-                return pos + 1
+            pos = self.prevChar(pos)
+            if self.bufChar(pos) == '\n':
+                return self.nextChar(pos)
         return 0
 
     def lineEnd(self, pos: int) -> int:
-        if pos < self.curPtr:
-            while pos < self.curPtr:
-                if self.buffer[pos] == ord('\n'):
-                    return pos
-                pos += 1
-            if self.curPtr == self.bufLen:
-                return self.bufLen
-        elif pos == self.bufLen:
-            return self.bufLen
-
-        while pos + self.gapLen < self.bufSize:
-            if self.buffer[pos + self.gapLen] == ord('\n'):
-                return pos
-            pos += 1
+        while pos < self.bufLen and self.bufChar(pos) != '\n':
+            pos = self.nextChar(pos)
         return pos
 
-    def formatLine(self, drawBuf: DrawBuffer, linePtr: int, width: int, color: int):
-        i = 0
-        pos = linePtr  # Buffer
+    def detectEol(self):
+        for p in range(self.bufLen):
+            ch = self.bufChar(p)
+            if ch == '\r':
+                if p + 1 < self.bufLen and self.bufChar(p + 1) == '\n':
+                    self.eolType = 0  # eolCrLf
+                else:
+                    self.eolType = 2  # eolCr
+                return
+            elif ch == '\n':
+                self.eolType = 1  # eolLf
+                return
+        # Default to CRLF
+        self.eolType = 0  # eolCrLf
 
-        while pos < self.curPtr and self.buffer[pos] != ord('\n') and i <= width:
-            i, pos = self.__highlight(color, drawBuf, i, pos, width)
+    def formatLine(self, drawBuf: DrawBuffer, linePtr: int, width: int, colors):
+        # The attributes for normal text are in the lower half of 'colors'.
+        # The attributes for text selection are in the upper half.
+        normal_attr, select_attr = colors.attrs
+        ranges = [
+            (normal_attr, self.selStart),
+            (select_attr, self.selEnd),
+            (normal_attr, self.bufLen)
+        ]
 
-        if pos >= self.curPtr:
-            pos += self.gapLen
-            while (pos < self.bufSize) and self.buffer[pos] != ord('\n') and i <= width:
-                i, pos = self.__highlight(color, drawBuf, i, pos, width)
+        P = linePtr
+        X = 0
+        last_color = normal_attr  # Track the actual color being used
 
-        for j in range(i, width):
-            if self.selStart <= pos < self.selEnd:
-                curColor = ((color & 0xFF00) >> 8) << DrawBuffer.CHAR_WIDTH
-            else:
-                curColor = (color & 0xFF) << DrawBuffer.CHAR_WIDTH
-            drawBuf[j] = 0x20 | curColor
+        # Use flag to implement goto fill behavior
+        should_fill = False
+
+        for current_color, end_pos in ranges:
+            if should_fill:
+                break
+
+            while P < end_pos:
+                last_color = current_color  # Track the color we're actually using
+                chars = self.bufChars(P)
+                char = chars[0]
+                if char == '\r' or char == '\n':
+                    # This is the goto fill - break out of everything
+                    should_fill = True
+                    break
+
+                if char == '\t':
+                    if X < width:
+                        # Fill to next tab stop (8-character intervals)
+                        while X % 8 != 0 and X < width:
+                            set_cell(drawBuf.data[X], ' ', current_color)
+                            X += 1
+                        if X < width:
+                            set_cell(drawBuf.data[X], ' ', current_color)
+                            X += 1
+                        P += 1
+                    else:
+                        break
+                else:
+                    result, new_X, new_P = self.formatCell(drawBuf.data, X, chars, P, current_color)
+                    if not result:
+                        break
+                    X = new_X
+                    P = new_P
+
+        # Fill remaining width with spaces using the last color we were actually using
+        while X < width:
+            set_cell(drawBuf.data[X], ' ', last_color)
+            X += 1
+
+    def formatCell(self, cells, width: int, text: str, p: int, color):
+        p_ = 0
+        w_ = width
+        success, w_, p_ = Text.draw_one(cells, w_, text, p_, color)
+        if success:
+            return True, w_, p + p_
+        return False, width, p
 
     def nextChar(self, pos: int) -> int:
-        if pos == self.bufLen:
-            return pos
-        return pos + 1
+        if pos + 1 < self.bufLen:
+            if self.bufChar(pos) == '\r' and self.bufChar(pos + 1) == '\n':
+                return pos + 2
+            if self.encoding == Encoding.encSingleByte:
+                return pos + 1
+            return pos + Text.next_char(self.bufChars(pos))
+        return self.bufLen
 
-    @staticmethod
-    def prevChar(pos):
-        if not pos:
-            return pos
-        return pos - 1
+    def prevChar(self, pos: int):
+        if pos > 1:
+            if self.bufChar(pos - 2) == '\r' and self.bufChar(pos - 1) == '\n':
+                return pos - 2
+            if self.encoding == Encoding.encSingleByte:
+                return pos - 1
+            t = self.prevBufChars(pos)
+            return pos - Text.prev_char(t, len(t))
+        return 0
 
-    def setBufSize(self, newSize):
+    def setBufSize(self, newSize: int):
+        """
+        Resize buffer - reconstruct string with larger gap.
+        """
         if newSize < 0x1000:
             # At least a 4K buffer
             newSize = 0x1000
@@ -842,20 +997,27 @@ class Editor(View):
             newSize = self.bufSize + 128
 
         if newSize > self.bufSize:
-            temp = self.buffer
-            nn = min(newSize, self.bufSize)
-            self.buffer = BufferArray(temp[:nn])
-            self.buffer.extend([0] * (newSize - nn))
-            n = self.bufLen - self.curPtr + self.delCount
-            self.buffer[newSize - n: newSize] = temp[self.bufSize - n: self.bufSize]
-            self.gapLen = self.bufSize - self.bufLen
+            # For string-based gap buffer, reconstruct with larger gap
+            # Get text before gap and after gap
+            before_gap = self.buffer[:self.curPtr]
+            after_gap_start = self.curPtr + self.gapLen
+            after_gap = self.buffer[after_gap_start:] if after_gap_start < len(self.buffer) else ''
+
+            # Calculate new gap size
+            new_gap_len = newSize - self.bufLen
+            gap_chars = '\x00' * new_gap_len  # Use null chars for gap
+
+            # Reconstruct buffer with larger gap
+            self.buffer = before_gap + gap_chars + after_gap
+            self.gapLen = new_gap_len
+
         return True
 
     @property
-    def bufSize(self):
+    def bufSize(self) -> int:
         return len(self.buffer)
 
-    def _handleEditorCommand(self, command, centerCursor, selectMode):
+    def _handleEditorCommand(self, command: int, centerCursor: bool, selectMode: int):
         self.lock()
         try:
             if command == cmCut:
@@ -915,11 +1077,15 @@ class Editor(View):
                 self.hideSelect()
             elif command == cmIndentMode:
                 self.autoIndent = not self.autoIndent
+            elif command == cmEncoding:
+                self.toggleEncoding()
+            elif command == cmSelectAll:
+                self.setSelect(0, self.bufLen, False)
             self.trackCursor(centerCursor)
         finally:
             self.unlock()
 
-    def _handleKeyDownEvent(self, centerCursor, event):
+    def _handleKeyDownEvent(self, centerCursor: bool, event: Event):
         self.lock()
         try:
             if self.overwrite and not self.hasSelection():
@@ -930,7 +1096,7 @@ class Editor(View):
         finally:
             self.unlock()
 
-    def _handleMouseEvent(self, event, selectMode):
+    def _handleMouseEvent(self, event: Event, selectMode: int):
         if event.mouse.eventFlags & meDoubleClick:
             selectMode |= smDouble
         done = False
@@ -960,22 +1126,3 @@ class Editor(View):
                 self.unlock()
 
             done = (not self.mouseEvent(event, evMouseMove + evMouseAuto))
-
-    def __highlight(self, color, drawBuf, i, pos, width):
-        if self.selStart <= pos < self.selEnd:
-            curColor = ((color & 0xFF00) >> 8) << DrawBuffer.CHAR_WIDTH
-        else:
-            curColor = (color & 0xFF) << DrawBuffer.CHAR_WIDTH
-        if self.buffer[pos] == ord('\t'):
-            _, r = divmod(i, self.tabSize)
-            for j in range(r):
-                drawBuf[i + j] = 0x20 | curColor
-                i += 1
-                if i + j > width:
-                    break
-            pos += 1
-        else:
-            drawBuf[i] = curColor | self.buffer[pos]
-            pos += 1
-            i += 1
-        return i, pos
