@@ -3,28 +3,30 @@ import os
 
 from vindauga.constants.event_codes import evBroadcast
 from vindauga.constants.std_dialog_commands import cmFileFocused
-from vindauga.misc.util import fexpand
+from vindauga.events.event import Event
+from vindauga.utilities.filesystem.path_utils import fexpand
 from vindauga.types.draw_buffer import DrawBuffer
 from vindauga.types.palette import Palette
+from vindauga.types.rect import Rect
 from vindauga.types.view import View
-from vindauga.misc.prefix.prefix import closestPrefix
+from vindauga.utilities.support.prefix.prefix import closestPrefix
 
 
 class FileInfoPane(View):
     cpInfoPane = "\x1E"
 
-    def __init__(self, bounds):
+    def __init__(self, bounds: Rect):
         super().__init__(bounds)
         self.eventMask |= evBroadcast
         self.__fileBlock = None
 
     @staticmethod
-    def getNiceSize(rate):
+    def getNiceSize(rate: float) -> str:
         if rate <= 0.0:
             return '0 B'
 
         value, prefix = closestPrefix(rate)
-        s = '{:.0f} {}B'.format(value, prefix)
+        s = f'{value:.0f} {prefix}B'
         return s
 
     def draw(self):
@@ -55,12 +57,11 @@ class FileInfoPane(View):
         b.moveChar(0, ' ', color, self.size.x)
         self.writeLine(0, 2, self.size.x, self.size.y - 2, b)
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         return Palette(self.cpInfoPane)
 
-    def handleEvent(self, event):
+    def handleEvent(self, event: Event):
         super().handleEvent(event)
         if event.what == evBroadcast and event.message.command == cmFileFocused:
             self.__fileBlock = event.message.infoPtr
             self.drawView()
-            self.clearEvent(event)
