@@ -1,6 +1,8 @@
 # -*- coding: utf-8 -*-
 from vindauga.constants.option_flags import ofFramed
-from vindauga.misc.util import clamp
+from vindauga.utilities.math_utils import clamp
+from vindauga.types.rect import Rect
+
 from .param_text import ParamText
 
 
@@ -12,17 +14,17 @@ class Gauge(ParamText):
 
     name = 'Gauge'
 
-    def __init__(self, bounds):
+    def __init__(self, bounds: Rect):
         super().__init__(bounds)
         self.options |= ofFramed
         self.currentValue = 0
         self.maxValue = 0
 
-    def setParams(self, value, maxValue):
+    def setParams(self, value: float, maxValue: float):
         self.maxValue = maxValue
         self.setValue(value)
 
-    def setValue(self, value):
+    def setValue(self, value: float):
         self.currentValue = clamp(value, 0, self.maxValue)
         fill = self.currentValue * self.size.x / self.maxValue
         dill = int(fill)
@@ -35,5 +37,5 @@ class Gauge(ParamText):
         buffer = buffer.ljust(self.size.x, self.BACK_CHAR)
         self.setText(buffer)
 
-    def getValue(self):
+    def getValue(self) -> float:
         return self.currentValue
