@@ -1,31 +1,36 @@
 # -*- coding: utf-8 -*-
 from gettext import gettext as _
 import logging
-import traceback
+from typing import Optional, Sequence
 
 from vindauga.constants.command_codes import cmScrollBarClicked, cmScrollBarChanged, cmListItemSelected
 from vindauga.constants.event_codes import evBroadcast, evKeyDown, evMouseDown, evMouseAuto, evMouseMove, meDoubleClick
-from vindauga.constants.keys import kbUp, kbDown, kbRight, kbLeft, kbPgDn, kbPgUp, kbHome, kbEnd, kbCtrlPgDn, kbCtrlPgUp, kbTab, kbShiftTab
+from vindauga.constants.keys import (kbUp, kbDown, kbRight, kbLeft, kbPgDn, kbPgUp, kbHome, kbEnd, kbCtrlPgDn,
+                                     kbCtrlPgUp, kbTab, kbShiftTab)
 
 from vindauga.constants.state_flags import sfSelected, sfActive
 from vindauga.constants.option_flags import ofFirstClick, ofSelectable
 from vindauga.events.event import Event
 
-from vindauga.misc.character_codes import SPECIAL_CHARS
-from vindauga.misc.message import message
-from vindauga.misc.util import ctrlToArrow
+from vindauga.utilities.input.character_codes import SPECIAL_CHARS
+from vindauga.utilities.message import message
+from vindauga.utilities.input.key_utils import ctrlToArrow
 from vindauga.types.draw_buffer import DrawBuffer
 from vindauga.types.palette import Palette
+from vindauga.types.rect import Rect
 from vindauga.types.view import View
+from vindauga.widgets.scroll_bar import ScrollBar
 
 logger = logging.getLogger(__name__)
+
 cmUpdateItemNumber = 901
 
 
 class GridView(View):
     cpGridView = '\x1A\x1A\x1B\x1C\x1D\x06'
 
-    def __init__(self, bounds, hScrollBar, vScrollBar, columnWidths):
+    def __init__(self, bounds: Rect, hScrollBar: Optional[ScrollBar], vScrollBar: Optional[ScrollBar],
+                 columnWidths: Sequence[int]):
         super().__init__(bounds)
         self.leftColumn = 0
         self.topRow = 0
@@ -92,8 +97,8 @@ class GridView(View):
                     text = self.getText(column, row, thisWidth)
                     b.moveStr(1, text, color)
                     if self.showMarkers:
-                        b.putChar(0, SPECIAL_CHARS[specialChar])
-                        b.putChar(thisWidth - 2, SPECIAL_CHARS[specialChar + 1])
+                        b.putChar(0, SPECIAL_CHARS[specialChar], color)
+                        b.putChar(thisWidth - 2, SPECIAL_CHARS[specialChar + 1], color)
                 elif not (i or column):
                     b.moveStr(1, _('<empty>'), self.getColor(1))
 
@@ -136,7 +141,7 @@ class GridView(View):
     def getPalette(self) -> Palette:
         return Palette(self.cpGridView)
 
-    def getText(self, _column, _row, _maxChars):
+    def getText(self, _column: int, _row: int, _maxChars: int) -> str:
         return ''
 
     def isSelected(self, column: int, row: int) -> bool:
@@ -250,7 +255,7 @@ class GridView(View):
                         self.focusItemNum(self.hScrollBar.value, self.vScrollBar.value)
                         self.drawView()
 
-    def selectItem(self, _column, _row):
+    def selectItem(self, _column: int, _row: int):
         message(self.owner, evBroadcast, cmListItemSelected, self)
 
     def setRange(self, columns: int, rows: int):
@@ -284,9 +289,8 @@ class GridView(View):
                     self.vScrollBar.hide()
             self.drawView()
 
-    def getColumnPosition(self, column: int):
+    def getColumnPosition(self, column: int) -> int:
         position = 0
-        # position = sum(self.columnWidths[self.leftColumn:max(self.numColumns, __column - 1)])
         for i in range(self.leftColumn, max(self.numColumns, column)):
             if i == column:
                 break
