@@ -4,6 +4,7 @@ from vindauga.constants.state_flags import sfDragging
 from vindauga.types.draw_buffer import DrawBuffer
 from vindauga.types.palette import Palette
 from vindauga.types.point import Point
+from vindauga.types.rect import Rect
 from vindauga.types.view import View
 
 
@@ -14,7 +15,7 @@ class Indicator(View):
     name = 'Indicator'
     cpIndicator = "\x02\x03"
 
-    def __init__(self, bounds):
+    def __init__(self, bounds: Rect):
         super().__init__(bounds)
         self._location = Point()
         self._modified = False
@@ -33,17 +34,18 @@ class Indicator(View):
         b.moveChar(0, frame, color, self.size.x)
 
         if self._modified:
-            b.putChar(0, '☼')
+            b.putChar(0, '☼', color)
 
-        s = ' {:3d}:{:3d} '.format(self._location.y + 1, self._location.x + 1)
+        s = f' {self._location.y + 1:3d}:{self._location.x + 1:3d} '
 
-        b.moveCStr(8 - s.find(':'), s, color)
+        attr_pair = color
+        b.moveCStr(8 - s.find(':'), s, attr_pair)
         self.writeBuf(0, 0, self.size.x, 1, b)
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         return Palette(self.cpIndicator)
 
-    def setState(self, state, enable):
+    def setState(self, state: int, enable: bool):
         super().setState(state, enable)
         if state == sfDragging:
             self.drawView()
