@@ -1,37 +1,39 @@
 # -*- coding: utf-8 -*-
 from dataclasses import dataclass
 import logging
+from typing import List, Optional
 
 from vindauga.constants.command_codes import wnNoNumber
 from vindauga.constants.option_flags import ofCentered, ofBuffered
 from vindauga.constants.event_codes import evBroadcast
-from vindauga.events.event_queue import EventQueue
-from vindauga.misc.message import message
+from vindauga.events.event import Event
+from vindauga.utilities.message import message
 from vindauga.types.palette import Palette
 from vindauga.types.rect import Rect
-from vindauga.utilities.draw_widget_tree import drawWidgetTree
+from vindauga.types.view import View
 
-from .message_window import cmFindInfoBox, cmFindMsgBox, cmInsInfoBox, cmInsMsgBox
+from .message_window import cmFindInfoBox, cmInsInfoBox
 from .program import Program
 from .static_prompt import StaticPrompt
 from .static_text import StaticText
 from .window import Window
 
+
 logger = logging.getLogger(__name__)
 
 
 class InfoWindow(Window):
     cpInfoWindow = "\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F\x14"
 
-    def __init__(self, bounds, title, windowNumber):
+    def __init__(self, bounds: Rect, title: str, windowNumber: int):
         super().__init__(bounds, title, windowNumber)
 
         self.flags = 0
         self.options |= (ofCentered | ofBuffered)
         self.count = 0
-        self.items = [None] * 25
+        self.items: List[Optional[View]] = [None] * 25
 
-    def handleEvent(self, event):
+    def handleEvent(self, event: Event):
         if event.what == evBroadcast:
             emc = event.message.command
             if emc == cmFindInfoBox:
@@ -67,17 +69,17 @@ class InfoWindow(Window):
                     self.unlock()
                     self.clearEvent(event)
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         return Palette(self.cpInfoWindow)
 
 
-@dataclass
+@dataclass(frozen=True)
 class InfoData:
     line: int
     text: str
 
 
-def postInfo(line, text):
+def postInfo(line: int, text: str):
     wPtr = message(Program.desktop, evBroadcast, cmFindInfoBox, 0)
     if line < 0 and wPtr:
         Program.desktop.destroy(wPtr)
