@@ -1,13 +1,19 @@
 # -*- coding: utf-8 -*-
+from typing import Optional
+
 from vindauga.constants.command_codes import (cmReceivedFocus, cmReleasedFocus)
 from vindauga.constants.event_codes import evBroadcast, evMouseDown, evKeyDown
 from vindauga.constants.option_flags import ofSelectable, ofPreProcess, ofPostProcess
 from vindauga.constants.state_flags import sfFocused
-from vindauga.misc.character_codes import SPECIAL_CHARS, getAltCode
-from vindauga.misc.util import hotKey
+from vindauga.events.event import Event
+from vindauga.utilities.input.character_codes import SPECIAL_CHARS, getAltCode
+from vindauga.utilities.text.string_utils import hotKey
 from vindauga.types.draw_buffer import DrawBuffer
-from vindauga.types.group import Group
+from vindauga.types.group import Phases
 from vindauga.types.palette import Palette
+from vindauga.types.rect import Rect
+from vindauga.types.view import View
+
 from .static_text import StaticText
 
 
@@ -15,7 +21,7 @@ class Label(StaticText):
     name = 'Label'
     cpLabel = "\x07\x08\x09\x09"
 
-    def __init__(self, bounds, text, linkedWidget=None):
+    def __init__(self, bounds: Rect, text: str, linkedWidget: Optional[View] = None):
         super().__init__(bounds, text)
         self._linkedWidget = linkedWidget
         self._light = False
@@ -35,36 +41,35 @@ class Label(StaticText):
 
         b.moveChar(0, ' ', color, self.size.x)
         if self._text:
-            b.moveCStr(1, self._text, color)
+            attr_pair = color
+            b.moveCStr(1, self._text, attr_pair)
 
         if self.showMarkers:
-            b.putChar(0, SPECIAL_CHARS[scOff])
+            b.putChar(0, SPECIAL_CHARS[scOff], color)
 
         self.writeLine(0, 0, self.size.x, 1, b)
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         palette = Palette(self.cpLabel)
         return palette
 
-    def focusLink(self, event):
+    def focusLink(self, event: Event):
         if self._linkedWidget and self._linkedWidget.options & ofSelectable:
             self._linkedWidget.focus()
         self.clearEvent(event)
 
-    def handleEvent(self, event):
+    def handleEvent(self, event: Event):
         super().handleEvent(event)
 
         if event.what == evMouseDown:
             self.focusLink(event)
-            self.clearEvent(event)
         elif event.what == evKeyDown:
             c = hotKey(self._text)
 
-            if (getAltCode(c) == event.keyDown.keyCode or c != 0 and self.owner.phase == Group.phPostProcess and
+            if (getAltCode(c) == event.keyDown.keyCode or c != 0 and self.owner.phase == Phases.Postprocess and
                     event.keyDown.charScan.charCode.upper() == c):
                 self.focusLink(event)
-                self.clearEvent(event)
-        elif event.what == evBroadcast and self._linkedWidget and event.message.command in {cmReceivedFocus, cmReleasedFocus}:
+        elif (event.what == evBroadcast and self._linkedWidget and
+              event.message.command in {cmReceivedFocus, cmReleasedFocus}):
             self._light = ((self._linkedWidget.state & sfFocused) != 0)
             self.drawView()
-            self.clearEvent(event)
