@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from vindauga.types.draw_buffer import DrawBuffer
 from vindauga.types.palette import Palette
+from vindauga.types.rect import Rect
 from vindauga.types.view import View
 
 
@@ -8,15 +9,15 @@ class MessageLine(View):
 
     cpMessageLine = "\x02\x04"
 
-    def __init__(self, bounds, s):
+    def __init__(self, bounds: Rect, s: str):
         super().__init__(bounds)
         self.width = bounds.bottomRight.x - bounds.topLeft.x
         self.text = s[:self.width]
 
-    def getPalette(self):
+    def getPalette(self) -> Palette:
         return Palette(self.cpMessageLine)
 
-    def setText(self, s):
+    def setText(self, s: str):
         self.text = s[:self.width]
         self.drawView()
 
