@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
+from vindauga.constants.grow_flags import gfGrowHiX, gfGrowHiY
 from vindauga.types.group import Group
-from vindauga.types.palette import Palette
+from vindauga.types.rect import Rect
 
 from .background import Background
 
@@ -9,14 +10,13 @@ class Panel(Group):
     """
     Group with a background installed. I do this enough that I'm making it a widget.
     """
-    cpPanel = "\x10\x11\x12\x12"
 
-    def __init__(self, bounds):
+    def __init__(self, bounds: Rect):
         super().__init__(bounds)
+        self.background = None
         self.initBackground()
 
     def initBackground(self):
-        self.insert(Background(self.getExtent(), ' '))
-
-    def getPalette(self):
-        return Palette(self.cpPanel)
+        self.background = Background(self.getExtent(), ' ')
+        self.background.growMode = gfGrowHiX | gfGrowHiY
+        self.insert(self.background)
