@@ -6,7 +6,7 @@ from .input_line import InputLine
 class PasswordInputLine(InputLine):
 
     def draw(self):
-        password = self.getDataString()
-        self.setData('*' * len(password))
+        password = self.current.data
+        self.current.data = list('*' * len(password))
         super().draw()
-        self.setData(password)
+        self.current.data = password
