@@ -1,34 +1,44 @@
 # -*- coding: utf-8 -*-
+from __future__ import annotations
 from gettext import gettext as _
 import logging
-import queue
-from queue import  Empty as QueueEmptyException
+import threading
+from typing import Any, Optional
 
-from vindauga.constants.command_codes import (cmReleasedFocus, cmCancel, cmSysRepaint, cmSysResize, cmSysWakeup,
-                                              cmSelectWindowNum, cmQuit, cmCommandSetChanged, cmMenu, cmClose, cmZoom,
-                                              cmResize, cmValid)
+from vindauga.constants.command_codes import (cmReleasedFocus, cmCancel, cmSelectWindowNum, cmQuit, cmCommandSetChanged, cmMenu, cmClose, cmZoom,
+                                              cmResize, cmValid, cmScreenChanged, cmTimerExpired)
 from vindauga.constants.event_codes import evNothing, evCommand, evKeyDown, evMouseDown, evBroadcast
-from vindauga.constants.keys import kbAltX, kbF10, kbCtrlW, kbF5, kbCtrlF5
+import vindauga.constants.keys
 from vindauga.constants.state_flags import sfVisible, sfSelected, sfFocused, sfModal, sfExposed
+import vindauga.constants.key_mappings as key_mappings
 from vindauga.events.event import Event
+from vindauga.events.event_queue import event_queue
 from vindauga.menus.menu_bar import MenuBar
-from vindauga.misc.character_codes import getAltChar
-from vindauga.misc.message import message
+from vindauga.utilities.input.character_codes import getAltChar
+from vindauga.utilities.message import message
+from vindauga.mouse.mouse import Mouse
+from vindauga.utilities.platform.system_interface import systemInterface
 from vindauga.types.display import Display
 from vindauga.types.group import Group
 from vindauga.types.palette import Palette
+from vindauga.types.point import Point
 from vindauga.types.rect import Rect
 from vindauga.types.screen import Screen
 from vindauga.types.status_def import StatusDef
 from vindauga.types.status_item import StatusItem
+from vindauga.types.timer_queue import TimerQueue
 from vindauga.types.view import View, SHADOW_SIZE
+
 from .desktop import Desktop
+from .dialog import Dialog
 from .status_line import StatusLine
+from .window import Window
+
 
 logger = logging.getLogger(__name__)
 
 
-def hasMouse(view, event):
+def hasMouse(view: View, event: Event) -> bool:
     return view.state & sfVisible and view.mouseInView(event.mouse.where)
 
 
@@ -44,8 +54,8 @@ class Program(Group):
     """
 
     exitText = _('~Alt+X~ Exit')
-    desktop = None
-    application = None
+    desktop: Optional[Desktop] = None
+    application: Optional['Application'] = None
     cpAppColor = "\x71\x70\x78\x74\x20\x28\x24\x17\x1F\x1A\x31\x31\x1E\x71\x1F" \
                  "\x37\x3F\x3A\x13\x13\x3E\x21\x3F\x70\x7F\x7A\x13\x13\x70\x7F\x7E" \
                  "\x70\x7F\x7A\x13\x13\x70\x70\x7F\x7E\x20\x2B\x2F\x78\x2E\x70\x30" \
@@ -76,25 +86,25 @@ class Program(Group):
                       "\x0F\x0F\x07\x70\x07\x07\x70\x07\x07\x07\x70\x0F\x07\x07\x01\x00" \
                       "\x07\x0F\x07\x70\x70\x07\x0F\x70"
 
-    color = Palette(cpAppColor)
-    blackWhite = Palette(cpAppBlackWhite)
-    monochrome = Palette(cpAppMonochrome)
-    palettes = [color, blackWhite, monochrome]
     apColor = 0
     apBlackWhite = 1
     apMonochrome = 2
-    pending = queue.Queue()
+    pending: Event = Event(evNothing)
+    timerQueue = TimerQueue()
+    eventTimeoutMs = 20
 
     def __init__(self):
-        w = Screen.screenWidth
-        h = Screen.screenHeight
+        w = systemInterface.getScreenCols()
+        h = systemInterface.getScreenRows()
         super().__init__(Rect(0, 0, w, h))
-
-        self.appPalette = self.apColor
+        color = Palette(self.cpAppColor)
+        blackWhite = Palette(self.cpAppBlackWhite)
+        monochrome = Palette(self.cpAppMonochrome)
+        self.palettes = [color, blackWhite, monochrome]
         self.initScreen()
         self.state = sfVisible | sfSelected | sfFocused | sfModal | sfExposed
         self.options = 0
-        self.buffer = Screen.screenBuffer
+        self.buffer = Screen.screen.screenBuffer
         self.statusLine = None
         self.menuBar = None
 
@@ -106,7 +116,6 @@ class Program(Group):
 
         self.menuBar = self.initMenuBar(self.getExtent())
         self.insert(self.menuBar)
-        Screen.doRepaint += 1
 
     def shutdown(self):
         self.statusLine = None
@@ -114,7 +123,7 @@ class Program(Group):
         Program.desktop = None
         super().shutdown()
 
-    def canMoveFocus(self):
+    def canMoveFocus(self) -> bool:
         """
         Returns True if the focus can be moved from one (desktop) view to another
         one.
@@ -125,7 +134,7 @@ class Program(Group):
         """
         return self.desktop.valid(cmReleasedFocus)
 
-    def executeDialog(self, pD, data):
+    def executeDialog(self, pD: Dialog, data) -> tuple[int, Any]:
         """
         Executes a dialog.
 
@@ -151,7 +160,7 @@ class Program(Group):
                 pD.setData(data)
             try:
                 c = Program.desktop.execView(pD)
-            except:
+            except Exception:
                 logger.exception('Dialog failed.')
                 c = cmCancel
             if c != cmCancel:
@@ -160,7 +169,15 @@ class Program(Group):
 
         return c, data
 
-    def getEvent(self, event):
+    def eventWaitTimeout(self):
+        timeout_ms = min(Program.timerQueue.timeUntilNextTimeout(), 2**64)
+        if timeout_ms < 0:
+            return self.eventTimeoutMs
+        if self.eventTimeoutMs < 0:
+            return timeout_ms
+        return min(timeout_ms, self.eventTimeoutMs)
+
+    def getEvent(self, event: Event):
         """
         Gets an event.
 
@@ -169,7 +186,7 @@ class Program(Group):
 
         `getEvent()` first checks if `Program.putEvent()` has generated a
         pending event. If so, `getEvent()` returns that event. If there is no
-        pending event, `getEvent()` calls `Screen.getEvent()`.
+        pending event, `getEvent()` calls `screen.getEvent()`.
 
         If both calls return `evNothing`, indicating that no user input is
         available, `getEvent()` calls `Program.idle()` to allow "background"
@@ -181,37 +198,29 @@ class Program(Group):
 
         :param event: Event object to be modified
         """
-        try:
-            event.setFrom(Program.pending.get_nowait())
-        except QueueEmptyException:
-            Screen.getEvent(event)
-            if event.what == evCommand:
-                c = event.message.command
-                if c == cmSysRepaint:
-                    self.redraw()
-                    self.clearEvent(event)
-                elif c == cmSysResize:
-                    Screen.curX = Screen.curY = 0
-                    self.buffer = Screen.screenBuffer
-                    self.changeBounds(Rect(0, 0,
-                                           Screen.screenWidth,
-                                           Screen.screenHeight))
+        if Program.pending.what != evNothing:
+            event.setFrom(Program.pending)
+            Program.pending.what = evNothing
+        else:
+            event_queue.waitForEvents(10)  # Reasonable timeout for event processing
+            event.getMouseEvent()
 
-                    self.setState(sfExposed, False)
-                    self.setState(sfExposed, True)
-                    self.redraw()
-                    self.clearEvent(event)
-                elif c == cmSysWakeup:
+            if event.what == evNothing:
+                event.getKeyEvent()
+                if event.what == evNothing:
                     self.idle()
-                    self.clearEvent(event)
 
         if self.statusLine:
-            if ((event.what & evKeyDown) or
-                (event.what & evMouseDown) and self.firstThat(self.hasMouse, event) is self.statusLine
-            ):
+            if event.what in (evKeyDown, evMouseDown) and self.firstThat(self.hasMouse, event) is self.statusLine:
                 self.statusLine.handleEvent(event)
 
-    def putEvent(self, event):
+        if event.what == evCommand and event.message.command == cmScreenChanged:
+            logger.info("Program received cmScreenChanged event - calling setScreenMode(smUpdate)")
+            self.setScreenMode(Display.smUpdate)
+            logger.info("setScreenMode(smUpdate) completed")
+            self.clearEvent(event)
+
+    def putEvent(self, event: Event):
         """
         Sets a pending event.
 
@@ -225,9 +234,9 @@ class Program(Group):
         """
         e = Event(evNothing)
         e.setFrom(event)
-        Program.pending.put(e)
+        Program.pending = e
 
-    def handleEvent(self, event):
+    def handleEvent(self, event: Event):
         """
         Standard `Program` event handler.
 
@@ -250,8 +259,6 @@ class Program(Group):
         """
         if event.what == evKeyDown:
             c = getAltChar(event.keyDown.keyCode)
-            if isinstance(c, int):
-                c = chr(c)
             if '1' <= c <= '9':
                 if self.canMoveFocus():
                     if message(Program.desktop, evBroadcast, cmSelectWindowNum, int(c)):
@@ -265,11 +272,15 @@ class Program(Group):
             self.endModal(cmQuit)
             self.clearEvent(event)
 
-    def getPalette(self):
-        return self.palettes[self.appPalette]
+    @staticmethod
+    def handleTimeout(id: TimerId, self):
+        message(self, evBroadcast, cmTimerExpired, id)
 
-    def setPalette(self, palette):
-        self.palettes[self.appPalette] = palette
+    def getPalette(self) -> Palette:
+        return self.appPalette
+
+    def setPalette(self, palette: Palette):
+        self.appPalette = Palette(palette)
 
     def idle(self):
         """
@@ -296,7 +307,9 @@ class Program(Group):
             message(self, evBroadcast, cmCommandSetChanged, None)
             View.commandSetChanged = False
 
-    def initDesktop(self, bounds):
+        self.timerQueue.collectExpiredTimers(self.handleTimeout, self)
+
+    def initDesktop(self, bounds: Rect):
         """
         Creates a new desktop.
        
@@ -326,26 +339,26 @@ class Program(Group):
         black & white). The shadows are usually painted in the right and bottom
         sides of menus and windows.
         """
-        global showMarkers
-        if Screen.screenMode & 0x00FF != Display.smMono:
-            if Screen.screenMode & Display.smFont8x8:
+        screen = Screen.screen
+        if screen.screenMode & 0x00FF != Display.smMono:
+            if screen.screenMode & Display.smFont8x8:
                 SHADOW_SIZE.x = 1
             else:
                 SHADOW_SIZE.x = 2
 
             SHADOW_SIZE.y = 1
-            showMarkers = False
-            if Screen.screenMode & 0x00FF == Display.smBW80:
-                self.appPalette = self.apBlackWhite
+            self.showMarkers = False
+            if screen.screenMode & 0x00FF == Display.smBW80:
+                self.appPalette = self.palettes[self.apBlackWhite]
             else:
-                self.appPalette = self.apColor
+                self.appPalette = self.palettes[self.apColor]
         else:
             SHADOW_SIZE.x = 0
             SHADOW_SIZE.y = 0
-            showMarkers = True
-            self.appPalette = self.apMonochrome
+            self.showMarkers = True
+            self.appPalette = self.palettes[self.apMonochrome]
 
-    def initMenuBar(self, bounds):
+    def initMenuBar(self, bounds: Rect):
         """
         Creates a new menu bar.
 
@@ -361,10 +374,10 @@ class Program(Group):
         bounds.bottomRight.y = bounds.topLeft.y + 1
         return MenuBar(bounds, [])
 
-    def initStatusLine(self, bounds):
+    def initStatusLine(self, bounds: Rect):
         """
         Creates a new status line.
-       
+
         This method creates a standard `StatusLine` view and returns it
 
         `initStatusLine()` should never be called directly. `initStatusLine()` is
@@ -376,13 +389,13 @@ class Program(Group):
         """
         bounds.topLeft.y = bounds.bottomRight.y - 1
         return StatusLine(bounds, StatusDef(0, 0xFFFF) +
-                          StatusItem(self.exitText, kbAltX, cmQuit) +
-                          StatusItem('', kbF10, cmMenu) +
-                          StatusItem('', kbCtrlW, cmClose) +
-                          StatusItem('', kbF5, cmZoom) +
-                          StatusItem('', kbCtrlF5, cmResize))
+                          StatusItem(self.exitText, vindauga.constants.keys.kbAltX, cmQuit) +
+                          StatusItem('', vindauga.constants.keys.kbF10, cmMenu) +
+                          StatusItem('', vindauga.constants.keys.kbCtrlW, cmClose) +
+                          StatusItem('', vindauga.constants.keys.kbF5, cmZoom) +
+                          StatusItem('', vindauga.constants.keys.kbCtrlF5, cmResize))
 
-    def insertWindow(self, window):
+    def insertWindow(self, window: Window):
         """
         Inserts a window in the `Program`.
 
@@ -409,19 +422,7 @@ class Program(Group):
         """
         self.execute()
 
-    def setScreenMode(self, *_args):
-        """
-        Resizes and redraws the screen.
-
-        :param _args: unused. Used to be screen mode.
-        """
-        r = Rect(0, 0, Screen.screenWidth, Screen.screenHeight)
-        self.changeBounds(r)
-        self.setState(sfExposed, False)
-        self.setState(sfExposed, True)
-        self.redraw()
-
-    def validView(self, view):
+    def validView(self, view: View):
         """
         Checks if a view is valid.
        
@@ -445,8 +446,26 @@ class Program(Group):
             return None
         return view
 
+    def killTimer(self, id: TimerId):
+        self.timerQueue.killTimer(id)
 
-def getDesktopSize():
+    def setScreenMode(self, mode):
+        Mouse.hide()
+        Screen.screen.setVideoMode(mode)
+        self.initScreen()
+        self.buffer = Screen.screen.screenBuffer
+        r = Rect(0, 0, Screen.screen.screenWidth, Screen.screen.screenHeight)
+        self.changeBounds(r)
+        self.setState(sfExposed, False)
+        self.setState(sfExposed, True)
+        self.redraw()
+        Mouse.show()
+
+    def setTimer(self, timeoutMs: int, periodMs: int) -> TimerId:
+        return self.timerQueue.setTimer(timeoutMs, periodMs)
+
+
+def getDesktopSize() -> Point:
     """
     Get the size of the desktop
 
