@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*_
+from vindauga.types.records.data_record import DataRecord
+
 from .cluster import Cluster
 
 
@@ -10,18 +12,18 @@ class RadioButtons(Cluster):
     def draw(self):
         self.drawMultiBox(self.button, ' ○')
 
-    def mark(self, item):
+    def mark(self, item: int) -> bool:
         return item == self._value
 
-    def press(self, item):
+    def press(self, item: int):
         self._value = item
 
-    def movedTo(self, item):
+    def movedTo(self, item: int):
         self._value = item
 
-    def setData(self, rec):
+    def setData(self, rec: DataRecord):
         super().setData(rec)
         self._sel = self._value
 
-    def consumesData(self):
+    def consumesData(self) -> bool:
         return True
